@@ -331,7 +331,7 @@ char *
 get_xdir_utf8 (void)
 {
 	if (!xdir_utf)	/* never free this, keep it for program life time */
-		xdir_utf = g_filename_to_utf8 (get_xdir_fs (), -1, 0, 0, 0);
+		xdir_utf = xchat_filename_to_utf8 (get_xdir_fs (), -1, 0, 0, 0);
 
 	return xdir_utf;
 }
@@ -421,6 +421,7 @@ const struct prefs vars[] = {
 	{"gui_dialog_top", P_OFFINT (dialog_top), TYPE_INT},
 	{"gui_dialog_width", P_OFFINT (dialog_width), TYPE_INT},
 	{"gui_hide_menu", P_OFFINT (hidemenu), TYPE_BOOL},
+	{"gui_input_spell", P_OFFINT (gui_input_spell), TYPE_BOOL},
 	{"gui_input_style", P_OFFINT (style_inputbox), TYPE_BOOL},
 	{"gui_join_dialog", P_OFFINT (gui_join_dialog), TYPE_BOOL},
 	{"gui_lagometer", P_OFFINT (lagometer), TYPE_INT},
@@ -437,6 +438,7 @@ const struct prefs vars[] = {
 	{"gui_ulist_size", P_OFFINT (paned_pos), TYPE_INT},
 	{"gui_ulist_sort", P_OFFINT (userlist_sort), TYPE_INT},
 	{"gui_ulist_style", P_OFFINT (style_namelistgad), TYPE_BOOL},
+	{"gui_url_mod", P_OFFINT (gui_url_mod), TYPE_INT},
 	{"gui_usermenu", P_OFFINT (gui_usermenu), TYPE_BOOL},
 	{"gui_win_height", P_OFFINT (mainwindow_height), TYPE_INT},
 	{"gui_win_left", P_OFFINT (mainwindow_left), TYPE_INT},
@@ -453,8 +455,8 @@ const struct prefs vars[] = {
 	{"input_beep_msg", P_OFFINT (beepmsg), TYPE_BOOL},
 	{"input_command_char", P_OFFSET (cmdchar), TYPE_STR},
 	{"input_filter_beep", P_OFFINT (filterbeep), TYPE_BOOL},
-	{"input_flash_hilight", P_OFFINT (flash_hilight), TYPE_BOOL},
-	{"input_fudge_snotice", P_OFFINT (fudgeservernotice), TYPE_BOOL},
+	{"input_flash_hilight", P_OFFINT (input_flash_hilight), TYPE_BOOL},
+	{"input_flash_priv", P_OFFINT (input_flash_priv), TYPE_BOOL},
 	{"input_perc_ascii", P_OFFINT (perc_ascii), TYPE_BOOL},
 	{"input_perc_color", P_OFFINT (perc_color), TYPE_BOOL},
 
@@ -494,6 +496,7 @@ const struct prefs vars[] = {
 	{"net_proxy_pass", P_OFFSET (proxy_pass), TYPE_STR},
 	{"net_proxy_port", P_OFFINT (proxy_port), TYPE_INT},
 	{"net_proxy_type", P_OFFINT (proxy_type), TYPE_INT},
+	{"net_proxy_use", P_OFFINT (proxy_use), TYPE_INT},
 	{"net_proxy_user", P_OFFSET (proxy_user), TYPE_STR},
 
 	{"net_reconnect_delay", P_OFFINT (recon_delay), TYPE_INT},
@@ -513,14 +516,13 @@ const struct prefs vars[] = {
 
 	{"tab_chans", P_OFFINT (tabchannels), TYPE_BOOL},
 	{"tab_dialogs", P_OFFINT (privmsgtab), TYPE_BOOL},
-	{"tab_dnd",  P_OFFINT (tab_dnd), TYPE_BOOL},
 	{"tab_icons", P_OFFINT (tab_icons), TYPE_BOOL},
 	{"tab_layout", P_OFFINT (tab_layout), TYPE_INT},
 	{"tab_new_to_front", P_OFFINT (newtabstofront), TYPE_INT},
 	{"tab_notices", P_OFFINT (notices_tabs), TYPE_BOOL},
 	{"tab_position", P_OFFINT (tabs_position), TYPE_INT},
 	{"tab_server", P_OFFINT (use_server_tab), TYPE_BOOL},
-	{"tab_small", P_OFFINT (tab_small), TYPE_BOOL},
+	{"tab_small", P_OFFINT (tab_small), TYPE_INT},
 	{"tab_sort", P_OFFINT (tab_sort), TYPE_BOOL},
 	{"tab_trunc", P_OFFINT (truncchans), TYPE_INT},
 	{"tab_utils", P_OFFINT (windows_as_tabs), TYPE_BOOL},
@@ -605,9 +607,11 @@ load_config (void)
 	prefs.wordwrap = 1;
 	prefs.autosave = 1;
 	prefs.autodialog = 1;
+	prefs.gui_input_spell = 1;
 	prefs.autoreconnect = 1;
 	prefs.recon_delay = 10;
 	prefs.tabchannels = 1;
+	prefs.tab_layout = 2;	/* 0=Tabs 1=Reserved 2=Tree */
 	prefs.tab_sort = 1;
 	prefs.paned_userlist = 1;
 	prefs.newtabstofront = 2;
@@ -643,10 +647,11 @@ load_config (void)
 	prefs.autoopendccsendwindow = 1;
 	prefs.autoopendccchatwindow = 1;
 	prefs.userhost = 1;
-	prefs.dcc_send_fillspaces = 1;
+	prefs.gui_url_mod = 4;	/* ctrl */
 	prefs.mainwindow_save = 1;
 	prefs.bantype = 2;
-	prefs.flash_hilight = 1;
+	prefs.input_flash_priv = prefs.input_flash_hilight = 1;
+	prefs.autodccsend = 2;	/* browse mode */
 #ifdef WIN32
 	prefs.identd = 1;
 #endif
@@ -763,7 +768,7 @@ save_config (void)
 		return 0;
 	}
 
-	if (!cfg_put_str (fh, "version", VERSION))
+	if (!cfg_put_str (fh, "version", PACKAGE_VERSION))
 	{
 		free (new_config);
 		return 0;
@@ -888,18 +893,18 @@ cmd_set (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 	int idx = 2;
 	char *var, *val;
 
-	if (strcasecmp (word[2], "-quiet") == 0)
-	{
-		idx++;
-		quiet = TRUE;
-	}
-
 	if (strcasecmp (word[2], "-e") == 0)
 	{
 		idx++;
 		erase = TRUE;
 	}
 
+	if (strcasecmp (word[idx], "-quiet") == 0)
+	{
+		idx++;
+		quiet = TRUE;
+	}
+
 	var = word[idx];
 	val = word_eol[idx+1];
 
@@ -976,3 +981,30 @@ cmd_set (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 
 	return TRUE;
 }
+
+int
+xchat_open_file (char *file, int flags, int mode, int xof_flags)
+{
+	char buf[1024];
+
+	if (xof_flags & XOF_FULLPATH)
+		return open (file, flags | OFLAGS);
+
+	snprintf (buf, sizeof (buf), "%s/%s", get_xdir_fs (), file);
+	if (xof_flags & XOF_DOMODE)
+		return open (buf, flags | OFLAGS, mode);
+	else
+		return open (buf, flags | OFLAGS);
+}
+
+FILE *
+xchat_fopen_file (const char *file, const char *mode, int xof_flags)
+{
+	char buf[1024];
+
+	if (xof_flags & XOF_FULLPATH)
+		return fopen (file, mode);
+
+	snprintf (buf, sizeof (buf), "%s/%s", get_xdir_fs (), file);
+	return fopen (buf, mode);
+}
