@@ -44,23 +44,6 @@
 #include "xchatc.h"
 
 
-/* black n white(0/1) are bad colors for nicks, and we'll use color 2 for us */
-/* also light/dark gray (14/15) */
-/* 5,7,8 are all shades of yellow which happen to look dman near the same */
-
-static char rcolors[] = { 19, 20, 22, 24, 25, 26, 27, 28, 29 };
-
-static int
-color_of (char *name)
-{
-	int i = 0, sum = 0;
-
-	while (name[i])
-		sum += name[i++];
-	sum %= sizeof (rcolors) / sizeof (char);
-	return rcolors[sum];
-}
-
 void
 clear_channel (session *sess)
 {
@@ -187,6 +170,9 @@ inbound_privmsg (server *serv, char *from, char *ip, char *text, int id)
 		if (prefs.beepmsg || (sess && sess->beep))
 			sound_beep (sess);
 
+		if (prefs.input_flash_priv)
+			fe_flash_window (sess);
+
 		if (ip && ip[0])
 		{
 			if (prefs.logging && sess->logfd != -1 &&
@@ -219,6 +205,9 @@ inbound_privmsg (server *serv, char *from, char *ip, char *text, int id)
 	if (prefs.beepmsg || sess->beep)
 		sound_beep (sess);
 
+	if (prefs.input_flash_priv)
+		fe_flash_window (sess);
+
 	if (sess->type == SESS_DIALOG)
 		EMIT_SIGNAL (XP_TE_DPRIVMSG, sess, from, text, idtext, NULL, 0);
 	else
@@ -366,6 +355,10 @@ inbound_action (session *sess, char *chan, char *from, char *text, int fromme, i
 		if (beep || sess->beep)
 			sound_beep (sess);
 
+		/* private action, flash? */
+		if (!is_channel (serv, chan) && prefs.input_flash_priv)
+			fe_flash_window (sess);
+
 		if (hilight)
 		{
 			EMIT_SIGNAL (XP_TE_HCHANACTION, sess, from, text, nickchar, NULL, 0);
@@ -373,15 +366,7 @@ inbound_action (session *sess, char *chan, char *from, char *text, int fromme, i
 		}
 	}
 
-	if (prefs.colorednicks)
-	{
-		char tbuf[NICKLEN + 4];
-		snprintf (tbuf, sizeof (tbuf), "\003%d%s", color_of (from), from);
-		EMIT_SIGNAL (XP_TE_CHANACTION, sess, tbuf, text, nickchar, NULL, 0);
-	} else
-	{
-		EMIT_SIGNAL (XP_TE_CHANACTION, sess, from, text, nickchar, NULL, 0);
-	}
+	EMIT_SIGNAL (XP_TE_CHANACTION, sess, from, text, nickchar, NULL, 0);
 }
 
 void
@@ -444,12 +429,6 @@ inbound_chanmsg (server *serv, session *sess, char *chan, char *from, char *text
 		EMIT_SIGNAL (XP_TE_DPRIVMSG, sess, from, text, idtext, NULL, 0);
 	else if (hilight)
 		EMIT_SIGNAL (XP_TE_HCHANMSG, sess, from, text, nickchar, idtext, 0);
-	else if (prefs.colorednicks)
-	{
-		char tbuf[NICKLEN + 4];
-		snprintf (tbuf, sizeof (tbuf), "\003%d%s", color_of (from), from);
-		EMIT_SIGNAL (XP_TE_CHANMSG, sess, tbuf, text, nickchar, idtext, 0);
-	}
 	else
 		EMIT_SIGNAL (XP_TE_CHANMSG, sess, from, text, nickchar, idtext, 0);
 }
@@ -802,8 +781,8 @@ inbound_ping_reply (session *sess, char *timestring, char *from)
 	if (lag)
 	{
 		sess->server->lag_sent = 0;
-		snprintf (outbuf, sizeof (outbuf), "%ld.%ld", dif / 100000, (dif / 10000) % 100);
-		fe_set_lag (sess->server, (int)((float)atof (outbuf)));
+		sess->server->lag = dif / 1000;
+		fe_set_lag (sess->server, dif / 100000);
 		return;
 	}
 
@@ -1244,9 +1223,14 @@ inbound_banlist (session *sess, time_t stamp, char *chan, char *mask, char *bann
 
 	sess = find_channel (serv, chan);
 	if (!sess)
+	{
 		sess = serv->front_session;
+		goto nowindow;
+	}
+
    if (!fe_is_banwindow (sess))
 	{
+nowindow:
 		/* let proto-irc.c do the 'goto def' for exemptions */
 		if (is_exemption)
 			return FALSE;
