@@ -79,9 +79,7 @@ void *xchat_realloc (char *old, int len, char *file, int line);
 #define CHANLEN		300
 #define PDIWORDS		32
 #define USERNAMELEN 10
-
-#define safe_strcpy(dest,src,len)	{strncpy((dest),(src),(len)); \
-												(dest)[len-1] = 0;}
+#define HIDDEN_CHAR	8			/* invisible character for xtext */
 
 #if defined(ENABLE_NLS) && !defined(_)
 #  include <libintl.h>
@@ -141,6 +139,7 @@ struct xchatprefs
 	char proxy_host[64];
 	int proxy_port;
 	int proxy_type; /* 0=disabled, 1=wingate 2=socks4, 3=socks5, 4=http */
+	int proxy_use; /* 0=all 1=IRC_ONLY 2=DCC_ONLY */
 	unsigned int proxy_auth;
 	char proxy_user[32];
 	char proxy_pass[32];
@@ -173,6 +172,7 @@ struct xchatprefs
 	int mainwindow_height;
 	int completion_sort;
 	int gui_win_state;
+	int gui_url_mod;
 	int gui_usermenu;
 	int gui_join_dialog;
 	int dialog_left;
@@ -188,7 +188,6 @@ struct xchatprefs
 	char dcc_ip_str[DOMAINLEN + 1];
 
 	unsigned int tab_small;
-	unsigned int tab_dnd;
 	unsigned int tab_sort;
 	unsigned int tab_icons;
 	unsigned int mainwindow_save;
@@ -230,7 +229,8 @@ struct xchatprefs
 	unsigned int beepmsg;
 	unsigned int beepchans;
 	unsigned int beephilight;
-	unsigned int flash_hilight;
+	unsigned int input_flash_hilight;
+	unsigned int input_flash_priv;
 	unsigned int truncchans;
 	unsigned int privmsgtab;
 	unsigned int irc_join_delay;
@@ -258,8 +258,8 @@ struct xchatprefs
 	unsigned int thin_separator;
 	unsigned int auto_indent;
 	unsigned int wordwrap;
+	unsigned int gui_input_spell;
 	unsigned int throttle;
-	unsigned int fudgeservernotice;
 	unsigned int topicbar;
 	unsigned int hideuserlist;
 	unsigned int hidemenu;
@@ -337,12 +337,21 @@ typedef struct session
 	int end_of_names:1;
 	int doing_who:1;		/* /who sent on this channel */
 	/* these are in the bottom-right menu */
-	int hide_join_part:1;	/* hide join & part messages? */
-	int beep:1;				/* beep enabled? */
-	int color_paste:1;
+	unsigned int hide_join_part:1;	/* hide join & part messages? */
+	unsigned int beep:1;				/* beep enabled? */
+	unsigned int color_paste:1;
 	int done_away_check:1;	/* done checking for away status changes */
+	unsigned int lastlog_regexp:1;	/* this is a lastlog and using regexp */
 } session;
 
+struct msproxy_state_t
+{
+	gint32				clientid;
+	gint32				serverid;
+	unsigned char		seq_recv;		/* seq number of last packet recv.	*/
+	unsigned char		seq_sent;		/* seq number of last packet sent.	*/
+};
+
 typedef struct server
 {
 	/*  server control operations (in server*.c) */
@@ -387,8 +396,12 @@ typedef struct server
 
 	int port;
 	int sok;					/* is equal to sok4 or sok6 (the one we are using) */
-	int sok4;				/* tcp4 socket */
-	int sok6;				/* tcp6 socket */
+	int sok4;					/* tcp4 socket */
+	int sok6;					/* tcp6 socket */
+	int proxy_sok;				/* Additional information for MS Proxy beast */
+	int proxy_sok4;
+	int proxy_sok6;
+	struct msproxy_state_t msp_state;
 	int id;					/* unique ID number (for plugin API) */
 #ifdef USE_OPENSSL
 	SSL *ssl;
@@ -404,7 +417,7 @@ typedef struct server
 	char servername[128];			/* what the server says is its name */
 	char password[86];
 	char nick[NICKLEN];
-	char linebuf[522];				/* RFC says 512 including \r\n */
+	char linebuf[2048];				/* RFC says 512 chars including \r\n */
 	char *last_away_reason;
 	int pos;								/* current position in linebuf */
 	int nickcount;
@@ -423,6 +436,7 @@ typedef struct server
 	time_t next_send;						/* cptr->since in ircu */
 	time_t prev_now;					/* previous now-time */
 	int sendq_len;						/* queue size */
+	int lag;								/* milliseconds */
 
 	struct session *front_session;	/* front-most window/tab */
 	struct session *server_session;	/* server window/tab */
@@ -462,6 +476,7 @@ typedef struct server
 	unsigned int have_idmsg:1;	/* freenode's IDENTIFY-MSG */
 	unsigned int have_except:1;	/* ban exemptions +e */
 	unsigned int using_cp1255:1;	/* encoding is CP1255/WINDOWS-1255? */
+	unsigned int using_irc:1;		/* encoding is "IRC" (CP1252/UTF-8 hybrid)? */
 	int use_who:1;				/* whether to use WHO command to get dcc_ip */
 #ifdef USE_OPENSSL
 	int use_ssl:1;					  /* is server SSL capable? */
@@ -478,7 +493,7 @@ struct commands
 	cmd_callback callback;
 	char needserver;
 	char needchannel;
-	char handle_quotes;
+	gint16 handle_quotes;
 	char *help;
 };
 
@@ -501,4 +516,12 @@ struct popup
 /* CL: get a random int in the range [0..n-1]. DON'T use rand() % n, it gives terrible results. */
 #define RAND_INT(n) ((int)(rand() / (RAND_MAX + 1.0) * (n)))
 
+#if defined(WIN32) && GLIB_CHECK_VERSION(2,4,0)
+#define xchat_filename_from_utf8 g_locale_from_utf8
+#define xchat_filename_to_utf8 g_locale_to_utf8
+#else
+#define xchat_filename_from_utf8 g_filename_from_utf8
+#define xchat_filename_to_utf8 g_filename_to_utf8
+#endif
+
 #endif
