@@ -56,6 +56,10 @@
 #include <gtk/gtkinvisible.h>
 #endif
 
+#ifdef USE_GTKSPELL
+#include <gtk/gtktextview.h>
+#endif
+
 #ifdef WIN32
 #include <windows.h>
 #endif
@@ -115,13 +119,14 @@ static gint arg_show_version = 0;
 
 static const GOptionEntry gopt_entries[] = 
 {
- {"no-auto",	'a', 0, G_OPTION_ARG_NONE,		&arg_dont_autoconnect, N_("Don't auto connect to servers"), NULL},
- {"cfgdir",		'd', 0, G_OPTION_ARG_STRING,	&arg_cfgdir, N_("Use a different config directory"), "PATH"},
- {"no-plugins",'n', 0, G_OPTION_ARG_NONE,		&arg_skip_plugins, N_("Don't auto load any plugins"), NULL},
- {"plugindir",	'p', 0, G_OPTION_ARG_NONE,		&arg_show_autoload, N_("Show plugin auto-load directory"), NULL},
- {"configdir",	'u', 0, G_OPTION_ARG_NONE,		&arg_show_config, N_("Show user config directory"), NULL},
- {"url",			 0,  0, G_OPTION_ARG_STRING,	&arg_url, N_("Open an irc://server:port/channel URL"), "URL"},
- {"version",	'v', 0, G_OPTION_ARG_NONE,		&arg_show_version, N_("Show version information"), NULL},
+ {"no-auto",	'a', 0, G_OPTION_ARG_NONE,	&arg_dont_autoconnect, N_("Don't auto connect to servers"), NULL},
+ {"cfgdir",	'd', 0, G_OPTION_ARG_STRING,	&arg_cfgdir, N_("Use a different config directory"), "PATH"},
+ {"no-plugins",	'n', 0, G_OPTION_ARG_NONE,	&arg_skip_plugins, N_("Don't auto load any plugins"), NULL},
+ {"plugindir",	'p', 0, G_OPTION_ARG_NONE,	&arg_show_autoload, N_("Show plugin auto-load directory"), NULL},
+ {"configdir",	'u', 0, G_OPTION_ARG_NONE,	&arg_show_config, N_("Show user config directory"), NULL},
+ {"url",	 0,  0, G_OPTION_ARG_STRING,	&arg_url, N_("Open an irc://server:port/channel URL"), "URL"},
+ {"existing",	'e', 0, G_OPTION_ARG_NONE,	&arg_existing, N_("Open URL in an existing XChat"), NULL},
+ {"version",	'v', 0, G_OPTION_ARG_NONE,	&arg_show_version, N_("Show version information"), NULL},
  {NULL}
 };
 
@@ -132,14 +137,14 @@ fe_args (int argc, char *argv[])
 	GOptionContext *context;
 
 #ifdef ENABLE_NLS
-	bindtextdomain (PACKAGE, LOCALEDIR);
-	bind_textdomain_codeset (PACKAGE, "UTF-8");
-	textdomain (PACKAGE);
+	bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);
+	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
+	textdomain (GETTEXT_PACKAGE);
 #endif
 
 	context = g_option_context_new (NULL);
-	g_option_context_add_main_entries (context, gopt_entries, PACKAGE);
-	g_option_context_add_group (context, gtk_get_option_group (TRUE));
+	g_option_context_add_main_entries (context, gopt_entries, GETTEXT_PACKAGE);
+	g_option_context_add_group (context, gtk_get_option_group (FALSE));
 	g_option_context_parse (context, &argc, &argv, &error);
 
 	if (error)
@@ -153,7 +158,7 @@ fe_args (int argc, char *argv[])
 
 	if (arg_show_version)
 	{
-		printf (PACKAGE" "VERSION"\n");
+		printf (PACKAGE_TARNAME" "PACKAGE_VERSION"\n");
 		return 0;
 	}
 
@@ -221,7 +226,11 @@ fe_args (int argc, char *argv[])
 const char cursor_color_rc[] =
 	"style \"xc-ib-st\""
 	"{"
+#ifdef USE_GTKSPELL
+		"GtkTextView::cursor-color=\"#%02x%02x%02x\""
+#else
 		"GtkEntry::cursor-color=\"#%02x%02x%02x\""
+#endif
 	"}"
 	"widget \"*.xchat-inputbox\" style : application \"xc-ib-st\"";
 
@@ -374,6 +383,8 @@ fe_message (char *msg, int flags)
 
 	dialog = gtk_message_dialog_new (GTK_WINDOW (parent_window), 0, type,
 												GTK_BUTTONS_OK, "%s", msg);
+	if (flags & FE_MSG_MARKUP)
+		gtk_message_dialog_set_markup (GTK_MESSAGE_DIALOG (dialog), msg);
 	g_signal_connect (G_OBJECT (dialog), "response",
 							G_CALLBACK (gtk_widget_destroy), 0);
 	gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
@@ -443,9 +454,10 @@ void
 fe_set_hilight (struct session *sess)
 {
 	if (sess->gui->is_tab)
-		fe_set_tab_color (sess, 3, TRUE);	/* blue, with taskbar flash */
-	else
-		fe_set_tab_color (sess, -1, TRUE);	/* taskbar flash only */
+		fe_set_tab_color (sess, 3);	/* set tab to blue */
+
+	if (prefs.input_flash_hilight)
+		fe_flash_window (sess); /* taskbar flash */
 }
 
 static void
@@ -501,12 +513,6 @@ fe_is_banwindow (struct session *sess)
    return 1;
 }
 
-void
-fe_chan_list_end (struct server *serv)
-{
-	gtk_widget_set_sensitive (serv->gui->chanlist_refresh, TRUE);
-}
-
 void
 fe_notify_update (char *name)
 {
@@ -570,9 +576,9 @@ fe_print_text (struct session *sess, char *text)
 	{
 		sess->new_data = TRUE;
 		if (sess->msg_said)
-			fe_set_tab_color (sess, 2, FALSE);
+			fe_set_tab_color (sess, 2);
 		else
-			fe_set_tab_color (sess, 1, FALSE);
+			fe_set_tab_color (sess, 1);
 	}
 }
 
@@ -582,36 +588,42 @@ fe_beep (void)
 	gdk_beep ();
 }
 
-typedef struct {
-	session *sess;
-	char *sstr;
-} fe_lastlog_info;
-
-static void
-fe_lastlog_foreach (GtkXText *xtext, unsigned char *text, void *data)
+#ifndef WIN32
+static int
+lastlog_regex_cmp (char *a, regex_t *reg)
 {
-	fe_lastlog_info *info = data;
-
-	if (nocasestrstr (text, info->sstr))
-		PrintText (info->sess, text);
+	return !regexec (reg, a, 1, NULL, REG_NOTBOL);
 }
+#endif
 
 void
-fe_lastlog (session *sess, session *lastlog_sess, char *sstr)
+fe_lastlog (session *sess, session *lastlog_sess, char *sstr, gboolean regexp)
 {
+#ifndef WIN32
+	regex_t reg;
+#endif
+
 	if (gtk_xtext_is_empty (sess->res->buffer))
 	{
 		PrintText (lastlog_sess, _("Search buffer is empty.\n"));
+		return;
 	}
-	else
+
+	if (!regexp)
 	{
-		fe_lastlog_info info;
-
-		info.sess = lastlog_sess;
-		info.sstr = sstr;
-
-		gtk_xtext_foreach (sess->res->buffer, fe_lastlog_foreach, &info);
+		gtk_xtext_lastlog (lastlog_sess->res->buffer, sess->res->buffer,
+								 (void *) nocasestrstr, sstr);
+		return;
 	}
+
+#ifndef WIN32
+	if (regcomp (&reg, sstr, REG_ICASE | REG_EXTENDED | REG_NOSUB) == 0)
+	{
+		gtk_xtext_lastlog (lastlog_sess->res->buffer, sess->res->buffer,
+								 (void *) lastlog_regex_cmp, &reg);
+		regfree (&reg);
+	}
+#endif
 }
 
 void
@@ -731,20 +743,43 @@ fe_ctrl_gui (session *sess, int action, int arg)
 	case 2:
 		mg_bring_tofront_sess (sess); break;
 	case 3:
-		fe_set_tab_color (sess, -1, TRUE); break; /* flash */
+		fe_flash_window (sess); break;
 	case 4:
-		fe_set_tab_color (sess, arg, FALSE); break;
+		fe_set_tab_color (sess, arg); break;
 	case 5:
 		gtk_window_iconify (GTK_WINDOW (sess->gui->window)); break;
 	case 6:
 		menu_bar_toggle ();	/* toggle menubar on/off */
 		break;
+	case 7:
+		mg_detach (sess, arg);	/* arg: 0=toggle 1=detach 2=attach */
+	}
+}
+
+static void
+dcc_saveas_cb (struct DCC *dcc, char *file)
+{
+	if (is_dcc (dcc))
+	{
+		if (dcc->dccstat == STAT_QUEUED)
+		{
+			if (file)
+				dcc_get_with_destfile (dcc, file);
+			else if (dcc->resume_sent == 0)
+				dcc_abort (dcc->serv->front_session, dcc);
+		}
 	}
 }
 
 void
 fe_confirm (const char *message, void (*yesproc)(void *), void (*noproc)(void *), void *ud)
 {
+	/* warning, assuming fe_confirm is used by DCC only! */
+	struct DCC *dcc = ud;
+
+	if (dcc->file)
+		gtkutil_file_req (message, dcc_saveas_cb, ud, dcc->file,
+								FRF_WRITE|FRF_FILTERISINITIAL|FRF_NOASKOVERWRITE);
 }
 
 int
@@ -793,7 +828,7 @@ fe_get_inputbox_contents (session *sess)
 		return sess->res->input_text;
 
 	/* current focused tab */
-	return GTK_ENTRY (sess->gui->input_box)->text;
+	return SPELL_ENTRY_GET_TEXT (sess->gui->input_box);
 }
 
 int
@@ -804,7 +839,7 @@ fe_get_inputbox_cursor (session *sess)
 		return 0;
 
 	/* current focused tab */
-	return gtk_editable_get_position (GTK_EDITABLE (sess->gui->input_box));
+	return SPELL_ENTRY_GET_POS (sess->gui->input_box);
 }
 
 void
@@ -813,8 +848,8 @@ fe_set_inputbox_cursor (session *sess, int delta, int pos)
 	if (!sess->gui->is_tab || sess == current_tab)
 	{
 		if (delta)
-			pos += gtk_editable_get_position (GTK_EDITABLE (sess->gui->input_box));
-		gtk_editable_set_position (GTK_EDITABLE (sess->gui->input_box), pos);
+			pos += SPELL_ENTRY_GET_POS (sess->gui->input_box);
+		SPELL_ENTRY_SET_POS (sess->gui->input_box, pos);
 	} else
 	{
 		/* we don't support changing non-front tabs yet */
@@ -826,12 +861,12 @@ fe_set_inputbox_contents (session *sess, char *text)
 {
 	if (!sess->gui->is_tab || sess == current_tab)
 	{
-		gtk_entry_set_text (GTK_ENTRY (sess->gui->input_box), text);
+		SPELL_ENTRY_SET_TEXT (sess->gui->input_box, text);
 	} else
 	{
-		if (sess->res->topic_text)
-			free (sess->res->topic_text);
-		sess->res->topic_text = strdup (text);
+		if (sess->res->input_text)
+			free (sess->res->input_text);
+		sess->res->input_text = strdup (text);
 	}
 }
 
@@ -843,8 +878,11 @@ fe_open_url_locale (const char *url)
 #else
 	char tbuf[512], *moz;
 
-	/* gnome 2.4+ has this */
-	moz = g_find_program_in_path ("gnome-open");
+	/* Think twice about patching this (yes you Debian!). Using gnome-open makes
+      it user friendly, by letting the user change their browser in Gnome:
+      System -> Preferences -> More Preferences -> Preferred Applications.
+      More info at http://xchat.org/faq/#q221 */
+	moz = g_find_program_in_path ("gnome-open"); /* Gnome 2.4+ has this */
 	if (moz)
 	{
 		snprintf (tbuf, sizeof (tbuf), "%s %s", moz, url);
@@ -853,7 +891,7 @@ fe_open_url_locale (const char *url)
 		return;
 	}
 
-	moz = g_find_program_in_path ("gnome-moz-remote");
+	moz = g_find_program_in_path ("firefox");
 	if (moz)
 	{
 		snprintf (tbuf, sizeof (tbuf), "%s %s", moz, url);
@@ -926,8 +964,21 @@ fe_server_event (server *serv, int type, int arg)
 				gtk_widget_set_sensitive (gui->menu_item[MENU_ID_AWAY], 0);
 				gtk_widget_set_sensitive (gui->menu_item[MENU_ID_DISCONNECT], 0);
 				gtk_widget_set_sensitive (gui->menu_item[MENU_ID_JOIN], 0);
+				/* close the join-dialog, if one exists */
+				joind (1, serv);
 			}
 		}
 		list = list->next;
 	}
 }
+
+void
+fe_get_file (const char *title, char *initial,
+				 void (*callback) (void *userdata, char *file), void *userdata,
+				 int flags)
+				
+{
+	/* OK: Call callback once per file, then once more with file=NULL. */
+	/* CANCEL: Call callback once with file=NULL. */
+	gtkutil_file_req (title, callback, userdata, initial, flags | FRF_FILTERISINITIAL);
+}
