@@ -58,6 +58,9 @@
 #include "textgui.h"
 #include "fkeys.h"
 
+#ifdef USE_GTKSPELL
+#include <gtk/gtktextview.h>
+#endif
 
 static void replace_handle (GtkWidget * wid);
 void key_action_tab_clean (void);
@@ -297,7 +300,7 @@ key_handle_key_press (GtkWidget *wid, GdkEventKey *evt, session *sess)
 			case 0:
 				return 1;
 			case 2:
-				gtk_signal_emit_stop_by_name (GTK_OBJECT (wid),
+				g_signal_stop_emission_by_name (G_OBJECT (wid),
 														"key_press_event");
 				return 1;
 			}
@@ -305,16 +308,24 @@ key_handle_key_press (GtkWidget *wid, GdkEventKey *evt, session *sess)
 		last = kb;
 		kb = kb->next;
 	}
-	if (keyval == GDK_space)
+
+	switch (keyval)
+	{
+	case GDK_space:
 		key_action_tab_clean ();
+		break;
 
-	/* check if it's a return or enter */
-	/* ---handled by the "activate" signal in maingui.c */
-/*	if ((evt->keyval == GDK_Return) || (evt->keyval == GDK_KP_Enter))
-		mg_inputbox_cb (wid, sess);
-
-	if (evt->keyval == GDK_KP_Enter)
-		gtk_signal_emit_stop_by_name (GTK_OBJECT (wid), "key_press_event");*/
+#if defined(USE_GTKSPELL) && !defined(WIN32)
+	/* gtktextview has no 'activate' event, so we trap ENTER here */
+	case GDK_Return:
+	case GDK_KP_Enter:
+		if (!(evt->state & GDK_CONTROL_MASK))
+		{
+			g_signal_stop_emission_by_name (G_OBJECT (wid), "key_press_event");
+			mg_inputbox_cb (wid, sess->gui);
+		}
+#endif
+	}
 
 	return 0;
 }
@@ -412,11 +423,9 @@ key_load_defaults ()
 		"CS\nPrior\nMove tab family left\nD1!\nD2!\n\n"\
 		"CS\nNext\nMove tab family right\nD1!\nD2!\n\n"\
 		"None\nF9\nRun Command\nD1:/GUI MENU TOGGLE\nD2!\n\n"
-	char buf[512];
 	int fd;
 
-	snprintf (buf, 512, "%s/keybindings.conf", get_xdir_fs ());
-	fd = open (buf, O_CREAT | O_TRUNC | O_WRONLY | OFLAGS, 0x180);
+	fd = xchat_open_file ("keybindings.conf", O_CREAT | O_TRUNC | O_WRONLY, 0x180, XOF_DOMODE);
 	if (fd < 0)
 		/* ???!!! */
 		return;
@@ -496,6 +505,16 @@ key_dialog_delete (GtkWidget * button, GtkCList * list)
 	}
 }
 
+static void
+key_print_text (GtkXText *xtext, char *text)
+{
+	unsigned int old = prefs.timestamp;
+	prefs.timestamp = 0;	/* temporarily disable stamps */
+	gtk_xtext_clear (GTK_XTEXT (xtext)->buffer);
+	PrintTextRaw (GTK_XTEXT (xtext)->buffer, text, 0);
+	prefs.timestamp = old;
+}
+
 static void
 key_dialog_sel_act (GtkWidget * un, int num)
 {
@@ -510,8 +529,7 @@ key_dialog_sel_act (GtkWidget * un, int num)
 								  _(key_actions[num].name));
 		if (key_actions[num].help)
 		{
-			gtk_xtext_clear (GTK_XTEXT (key_dialog_text)->buffer);
-			PrintTextRaw (GTK_XTEXT (key_dialog_text)->buffer, _(key_actions[num].help), 0);
+			key_print_text (GTK_XTEXT (key_dialog_text), _(key_actions[num].help));
 		}
 	}
 }
@@ -533,8 +551,7 @@ key_dialog_sel_row (GtkWidget * clist, gint row, gint column,
 											  kb->action);
 		if (key_actions[kb->action].help)
 		{
-			gtk_xtext_clear (GTK_XTEXT (key_dialog_text)->buffer);
-			PrintTextRaw (GTK_XTEXT (key_dialog_text)->buffer, _(key_actions[kb->action].help), 0);
+			key_print_text (GTK_XTEXT (key_dialog_text), _(key_actions[kb->action].help));
 		}
 	}
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (key_dialog_tog_c),
@@ -641,7 +658,7 @@ key_dialog_set_key (GtkWidget * entry, GdkEventKey * evt, void *none)
 	kb->keyname = key_get_key_name (kb->keyval);
 	gtk_clist_set_text (GTK_CLIST (key_dialog_kb_clist), row, 1, kb->keyname);
 	gtk_entry_set_text (GTK_ENTRY (entry), kb->keyname);
-	gtk_signal_emit_stop_by_name (GTK_OBJECT (entry), "key_press_event");
+	g_signal_stop_emission_by_name (G_OBJECT (entry), "key_press_event");
 }
 
 static void
@@ -825,12 +842,11 @@ key_save_kbs (char *fn)
 	struct key_binding *kb;
 
 	if (!fn)
-		snprintf (buf, 510, "%s/keybindings.conf", get_xdir_fs ());
+		fd = xchat_open_file ("keybindings.conf", O_CREAT | O_TRUNC | O_WRONLY,
+									 0x180, XOF_DOMODE);
 	else
-	{
-		safe_strcpy (buf, fn, sizeof (buf));
-	}
-	fd = open (buf, O_CREAT | O_TRUNC | O_WRONLY | OFLAGS, 0x180);
+		fd = xchat_open_file (fn, O_CREAT | O_TRUNC | O_WRONLY,
+									 0x180, XOF_DOMODE | XOF_FULLPATH);
 	if (fd < 0)
 	{
 		fe_message (_("Error opening keys config file\n"), FE_MSG_ERROR);
@@ -953,14 +969,10 @@ key_load_kbs (char *filename)
 	struct key_binding *kb = NULL, *last = NULL;
 	int fd, len, pnt = 0, state = 0, n;
 
-	buf = malloc (1000);
 	if (filename == NULL)
-		snprintf (buf, 1000, "%s/keybindings.conf", get_xdir_fs ());
+		fd = xchat_open_file ("keybindings.conf", O_RDONLY, 0, 0);
 	else
-		strcpy (buf, filename);
-
-	fd = open (buf, O_RDONLY | OFLAGS);
-	free (buf);
+		fd = xchat_open_file (filename, O_RDONLY, 0, XOF_FULLPATH);
 	if (fd < 0)
 		return 1;
 	if (fstat (fd, &st) != 0)
@@ -1227,9 +1239,9 @@ key_action_insert (GtkWidget * wid, GdkEventKey * evt, char *d1, char *d2,
 	if (!d1)
 		return 1;
 
-	tmp_pos = gtk_editable_get_position (GTK_EDITABLE (wid));
-	gtk_editable_insert_text (GTK_EDITABLE (wid), d1, strlen (d1), &tmp_pos);
-	gtk_editable_set_position (GTK_EDITABLE (wid), tmp_pos);
+	tmp_pos = SPELL_ENTRY_GET_POS (wid);
+	SPELL_ENTRY_INSERT (wid, d1, strlen (d1), &tmp_pos);
+	SPELL_ENTRY_SET_POS (wid, tmp_pos);
 	return 2;
 }
 
@@ -1297,8 +1309,8 @@ key_action_set_buffer (GtkWidget * wid, GdkEventKey * evt, char *d1, char *d2,
 	if (d1[0] == 0)
 		return 1;
 
-	gtk_entry_set_text (GTK_ENTRY (wid), d1);
-	gtk_editable_set_position (GTK_EDITABLE (wid), -1);
+	SPELL_ENTRY_SET_TEXT (wid, d1);
+	SPELL_ENTRY_SET_POS (wid, -1);
 
 	return 2;
 }
@@ -1309,11 +1321,11 @@ key_action_history_up (GtkWidget * wid, GdkEventKey * ent, char *d1, char *d2,
 {
 	char *new_line;
 
-	new_line = history_up (&sess->history, (char *)GTK_ENTRY (wid)->text);
+	new_line = history_up (&sess->history, SPELL_ENTRY_GET_TEXT (wid));
 	if (new_line)
 	{
-		gtk_entry_set_text (GTK_ENTRY (wid), new_line);
-		gtk_editable_set_position (GTK_EDITABLE (wid), -1);
+		SPELL_ENTRY_SET_TEXT (wid, new_line);
+		SPELL_ENTRY_SET_POS (wid, -1);
 	}
 
 	return 2;
@@ -1328,8 +1340,8 @@ key_action_history_down (GtkWidget * wid, GdkEventKey * ent, char *d1,
 	new_line = history_down (&sess->history);
 	if (new_line)
 	{
-		gtk_entry_set_text (GTK_ENTRY (wid), new_line);
-		gtk_editable_set_position (GTK_EDITABLE (wid), -1);
+		SPELL_ENTRY_SET_TEXT (wid, new_line);
+		SPELL_ENTRY_SET_POS (wid, -1);
 	}
 
 	return 2;
@@ -1419,16 +1431,16 @@ key_action_tab_comp (GtkWidget *t, GdkEventKey *entry, char *d1, char *d2,
 	GCompletion *gcomp = NULL;
 
 	/* force the IM Context to reset */
-	gtk_editable_set_editable (GTK_EDITABLE (t), FALSE);
-	gtk_editable_set_editable (GTK_EDITABLE (t), TRUE);
+	SPELL_ENTRY_SET_EDITABLE (t, FALSE);
+	SPELL_ENTRY_SET_EDITABLE (t, TRUE);
 
-	text = GTK_ENTRY (t)->text;
+	text = SPELL_ENTRY_GET_TEXT (t);
 	if (text[0] == 0)
 		return 1;
 
 	len = g_utf8_strlen (text, -1); /* must be null terminated */
 
-	cursor_pos = gtk_editable_get_position (GTK_EDITABLE (t));
+	cursor_pos = SPELL_ENTRY_GET_POS (t);
 
 	buf[0] = 0; /* make sure we don't get garbage in the buffer */
 
@@ -1611,8 +1623,8 @@ key_action_tab_comp (GtkWidget *t, GdkEventKey *entry, char *d1, char *d2,
 							strcat (buf, " ");
 							strncat (buf, postfix, COMP_BUF - cursor_pos -1);
 						}
-						gtk_entry_set_text (GTK_ENTRY (t), buf);
-						gtk_editable_set_position (GTK_EDITABLE (t), g_utf8_pointer_to_offset(buf, buf + cursor_pos));
+						SPELL_ENTRY_SET_TEXT (t, buf);
+						SPELL_ENTRY_SET_POS (t, g_utf8_pointer_to_offset(buf, buf + cursor_pos));
 						buf[0] = 0;
 					}
 					else
@@ -1653,8 +1665,8 @@ key_action_tab_comp (GtkWidget *t, GdkEventKey *entry, char *d1, char *d2,
 		cursor_pos = strlen (buf);
 		if (postfix)
 			strncat (buf, postfix, COMP_BUF - cursor_pos - 2);
-		gtk_entry_set_text (GTK_ENTRY (t), buf);
-		gtk_editable_set_position (GTK_EDITABLE (t), g_utf8_pointer_to_offset(buf, buf + cursor_pos));
+		SPELL_ENTRY_SET_TEXT (t, buf);
+		SPELL_ENTRY_SET_POS (t, g_utf8_pointer_to_offset(buf, buf + cursor_pos));
 	}
 	if (gcomp)
 		g_completion_free(gcomp);
@@ -1716,8 +1728,8 @@ static int
 key_action_put_history (GtkWidget * wid, GdkEventKey * ent, char *d1,
 									char *d2, struct session *sess)
 {
-	history_add (&sess->history, (char *)GTK_ENTRY (wid)->text);
-	gtk_entry_set_text (GTK_ENTRY (wid), "");
+	history_add (&sess->history, SPELL_ENTRY_GET_TEXT (wid));
+	SPELL_ENTRY_SET_TEXT (wid, "");
 	return 2;						  /* -''- */
 }
 
@@ -1740,7 +1752,7 @@ replace_handle (GtkWidget *t)
 	char outbuf[4096];
 	int c, len, xlen;
 
-	text = gtk_entry_get_text (GTK_ENTRY (t));
+	text = SPELL_ENTRY_GET_TEXT (t);
 
 	len = strlen (text);
 	if (len < 1)
@@ -1792,8 +1804,8 @@ replace_handle (GtkWidget *t)
 			else
 				snprintf (word, sizeof (word), "%s%s", pop->cmd, postfix);
 			strcat (outbuf, word);
-			gtk_entry_set_text (GTK_ENTRY (t), outbuf);
-			gtk_editable_set_position (GTK_EDITABLE (t), -1);
+			SPELL_ENTRY_SET_TEXT (t, outbuf);
+			SPELL_ENTRY_SET_POS (t, -1);
 			return;
 		}
 		list = list->next;
