@@ -44,13 +44,11 @@
 #include <gtk/gtkcellrenderertext.h>
 #include <gtk/gtkcellrenderertoggle.h>
 #include <gtk/gtkversion.h>
-#if GTK_CHECK_VERSION(2,4,0)
 #include <gtk/gtkfilechooserdialog.h>
-#else
-#include <gtk/gtkfilesel.h>
-#endif
 
 #include "../common/xchat.h"
+#include "../common/fe.h"
+#include "../common/util.h"
 #include "gtkutil.h"
 #include "pixmaps.h"
 
@@ -64,28 +62,12 @@ struct file_req
 	GtkWidget *dialog;
 	void *userdata;
 	filereqcallback callback;
-	int write;
+	int flags;		/* FRF_* flags */
 };
 
 static char last_dir[256] = "";
 
 
-GtkWidget *
-gtkutil_simpledialog (char *msg)
-{
-	GtkWidget *dialog;
-
-	dialog = gtk_message_dialog_new (NULL, 0, GTK_MESSAGE_INFO, GTK_BUTTONS_OK,
-						 "%s", msg);
-	g_signal_connect (G_OBJECT (dialog), "response",
-							G_CALLBACK (gtk_widget_destroy), 0);
-	gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
-	gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_MOUSE);
-	gtk_widget_show (dialog);
-
-	return dialog;
-}
-
 static void
 gtkutil_file_req_destroy (GtkWidget * wid, struct file_req *freq)
 {
@@ -101,7 +83,7 @@ gtkutil_check_file (char *file, struct file_req *freq)
 
 	path_part (file, last_dir, sizeof (last_dir));
 
-	if (stat (file, &st) != -1)
+	if (!(freq->flags & FRF_CHOOSEFOLDER) && stat (file, &st) != -1)
 	{
 		if (S_ISDIR (st.st_mode))
 		{
@@ -111,17 +93,12 @@ gtkutil_check_file (char *file, struct file_req *freq)
 
 			if (tmp[strlen(tmp)-1] != '/')
 				strcat (tmp, "/");
-#if GTK_CHECK_VERSION(2,4,0)
 			gtk_file_chooser_set_filename (GTK_FILE_CHOOSER (freq->dialog), tmp);
-#else
-			gtk_file_selection_set_filename (GTK_FILE_SELECTION (freq->dialog),
-														tmp);
-#endif
 			free (tmp);
 			return 0;
 		}
 	}
-	if (freq->write)
+	if (freq->flags & FRF_WRITE)
 	{
 		if (access (last_dir, W_OK) == 0)
 			axs = TRUE;
@@ -129,7 +106,7 @@ gtkutil_check_file (char *file, struct file_req *freq)
 	{
 		if (stat (file, &st) != -1)
 		{
-			if (!S_ISDIR (st.st_mode))
+			if (!S_ISDIR (st.st_mode) || (freq->flags & FRF_CHOOSEFOLDER))
 				axs = TRUE;
 		}
 	}
@@ -139,21 +116,21 @@ gtkutil_check_file (char *file, struct file_req *freq)
 		char *utf8_file;
 		/* convert to UTF8. It might be converted back to locale by
 			server.c's g_convert */
-		utf8_file = g_filename_to_utf8 (file, -1, NULL, NULL, NULL);
+		utf8_file = xchat_filename_to_utf8 (file, -1, NULL, NULL, NULL);
 		if (utf8_file)
 		{
 			freq->callback (freq->userdata, utf8_file);
 			g_free (utf8_file);
 		} else
 		{
-			gtkutil_simpledialog ("Filename encoding is corrupt.");
+			fe_message ("Filename encoding is corrupt.", FE_MSG_ERROR);
 		}
 	} else
 	{
-		if (freq->write)
-			gtkutil_simpledialog (_("Cannot write to that file."));
+		if (freq->flags & FRF_WRITE)
+			fe_message (_("Cannot write to that file."), FE_MSG_ERROR);
 		else
-			gtkutil_simpledialog (_("Cannot read that file."));
+			fe_message (_("Cannot read that file."), FE_MSG_ERROR);
 	}
 
 	return 1;
@@ -162,12 +139,11 @@ gtkutil_check_file (char *file, struct file_req *freq)
 static void
 gtkutil_file_req_done (GtkWidget * wid, struct file_req *freq)
 {
-#if GTK_CHECK_VERSION(2,4,0)
 	int kill = 0;
 	GSList *files, *cur;
 	GtkFileChooser *fs = GTK_FILE_CHOOSER (freq->dialog);
 
-	if (gtk_file_chooser_get_select_multiple (fs))
+	if (freq->flags & FRF_MULTIPLE)
 	{
 		files = cur = gtk_file_chooser_get_filenames (fs);
 		while (cur)
@@ -180,37 +156,16 @@ gtkutil_file_req_done (GtkWidget * wid, struct file_req *freq)
 			g_slist_free (files);
 	} else
 	{
-		kill = gtkutil_check_file (gtk_file_chooser_get_filename (fs), freq);
+		if (freq->flags & FRF_CHOOSEFOLDER)
+			kill = gtkutil_check_file (gtk_file_chooser_get_current_folder (fs), freq);
+		else
+			kill = gtkutil_check_file (gtk_file_chooser_get_filename (fs), freq);
 	}
-#else
-	int i = 0, kill = 0;
-	gchar **files;
-	GtkFileSelection *fs = GTK_FILE_SELECTION (freq->dialog);
-
-	if (gtk_file_selection_get_select_multiple (fs))
-	{
-		files = gtk_file_selection_get_selections (fs);
-		if (files)
-		{
-			while (files[i])
-			{
-				kill |= gtkutil_check_file (files[i], freq);
-				i++;
-			}
-			g_strfreev (files);
-		}
-	} else
-	{
-		kill |= gtkutil_check_file ((char *)gtk_file_selection_get_filename (fs), freq);
-	}
-#endif
 
 	if (kill)	/* this should call the "destroy" cb, where we free(freq) */
 		gtk_widget_destroy (freq->dialog);
 }
 
-#if GTK_CHECK_VERSION(2,4,0)
-
 static void
 gtkutil_file_req_response (GtkWidget *dialog, gint res, struct file_req *freq)
 {
@@ -226,31 +181,33 @@ gtkutil_file_req_response (GtkWidget *dialog, gint res, struct file_req *freq)
 	}
 }
 
-#else
-
-static void
-gtkutil_file_req_cancel (GtkWidget * wid, struct file_req *freq)
-{
-	gtk_widget_destroy (freq->dialog);
-}
-
-#endif
- 
 void
-gtkutil_file_req (char *title, void *callback, void *userdata, char *filter,
+gtkutil_file_req (const char *title, void *callback, void *userdata, char *filter,
 						int flags)
 {
 	struct file_req *freq;
 	GtkWidget *dialog;
+	extern char *get_xdir_fs (void);
 
-#if GTK_CHECK_VERSION(2,4,0)
-	/* == GTK 2.4 uses the new GtkFileChooser, which is much nicer! */
 	if (flags & FRF_WRITE)
+	{
 		dialog = gtk_file_chooser_dialog_new (title, NULL,
 												GTK_FILE_CHOOSER_ACTION_SAVE,
 												GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
 												GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
 												NULL);
+		if (filter && filter[0])	/* filter becomes initial name when saving */
+		{
+			char temp[1024];
+			path_part (filter, temp, sizeof (temp));
+			gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (dialog), temp);
+			gtk_file_chooser_set_current_name (GTK_FILE_CHOOSER (dialog), file_part (filter));
+		}
+#if GTK_CHECK_VERSION(2,8,0)
+		if (!(flags & FRF_NOASKOVERWRITE))
+			gtk_file_chooser_set_do_overwrite_confirmation (GTK_FILE_CHOOSER (dialog), TRUE);
+#endif
+	}
 	else
 		dialog = gtk_file_chooser_dialog_new (title, NULL,
 												GTK_FILE_CHOOSER_ACTION_OPEN,
@@ -263,39 +220,26 @@ gtkutil_file_req (char *title, void *callback, void *userdata, char *filter,
 		gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (dialog), last_dir);
 	if (flags & FRF_ADDFOLDER)
 		gtk_file_chooser_add_shortcut_folder (GTK_FILE_CHOOSER (dialog),
-														  (char *)get_xdir_fs (), NULL);
+														  get_xdir_fs (), NULL);
+	if (flags & FRF_CHOOSEFOLDER)
+	{
+		gtk_file_chooser_set_action (GTK_FILE_CHOOSER (dialog), GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER);
+		gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (dialog), filter);
+	}
+	else
+	{
+		if (filter && (flags & FRF_FILTERISINITIAL))
+			gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (dialog), filter);
+	}
 
 	freq = malloc (sizeof (struct file_req));
 	freq->dialog = dialog;
-	freq->write = (flags & FRF_WRITE);
+	freq->flags = flags;
 	freq->callback = callback;
 	freq->userdata = userdata;
 
 	g_signal_connect (G_OBJECT (dialog), "response",
 							G_CALLBACK (gtkutil_file_req_response), freq);
-
-#else
-	dialog = gtk_file_selection_new (title);
-	if (flags & FRF_MULTIPLE)
-		gtk_file_selection_set_select_multiple (GTK_FILE_SELECTION (dialog), TRUE);
-	if (last_dir[0])
-		gtk_file_selection_set_filename (GTK_FILE_SELECTION (dialog), last_dir);
-
-	freq = malloc (sizeof (struct file_req));
-	freq->dialog = dialog;
-	freq->write = (flags & FRF_WRITE);
-	freq->callback = callback;
-	freq->userdata = userdata;
-
-	g_signal_connect (G_OBJECT
-							  (GTK_FILE_SELECTION (dialog)->cancel_button),
-							  "clicked", G_CALLBACK (gtkutil_file_req_cancel),
-							  (gpointer) freq);
-	g_signal_connect (G_OBJECT (GTK_FILE_SELECTION (dialog)->ok_button),
-							  "clicked", G_CALLBACK (gtkutil_file_req_done),
-							  (gpointer) freq);
-#endif
-
 	g_signal_connect (G_OBJECT (dialog), "destroy",
 						   G_CALLBACK (gtkutil_file_req_destroy), (gpointer) freq);
 	gtk_widget_show (dialog);
@@ -439,37 +383,34 @@ fe_get_int (char *msg, int def, void *callback, void *userdata)
 
 GtkWidget *
 gtkutil_button (GtkWidget *box, char *stock, char *tip, void *callback,
-				 void *userdata, char *labeltext)
+					 void *userdata, char *labeltext)
 {
-	GtkWidget *wid, *img, *label, *bbox;
+	GtkWidget *wid, *img, *bbox;
 
 	wid = gtk_button_new ();
-	g_signal_connect (G_OBJECT (wid), "clicked",
-							G_CALLBACK (callback), userdata);
-
-	bbox = gtk_hbox_new (0, 0);
-	gtk_container_add (GTK_CONTAINER (wid), bbox);
-	gtk_widget_show (bbox);
-
-	if (stock)
-	{
-		img = gtk_image_new_from_stock (stock, GTK_ICON_SIZE_MENU);
-		gtk_widget_set_usize (img, 8, 6);
-		gtk_container_add (GTK_CONTAINER (bbox), img);
-		gtk_widget_show (img);
-	}
 
 	if (labeltext)
 	{
-		label = gtk_label_new_with_mnemonic (labeltext);
-		gtk_container_add (GTK_CONTAINER (bbox), label);
-		gtk_widget_show (label);
+		gtk_button_set_label (GTK_BUTTON (wid), labeltext);
+		gtk_button_set_image (GTK_BUTTON (wid), gtk_image_new_from_stock (stock, GTK_ICON_SIZE_MENU));
 		gtk_container_add (GTK_CONTAINER (box), wid);
-	} else
+	}
+	else
 	{
+		bbox = gtk_hbox_new (0, 0);
+		gtk_container_add (GTK_CONTAINER (wid), bbox);
+		gtk_widget_show (bbox);
+
+		img = gtk_image_new_from_stock (stock, GTK_ICON_SIZE_MENU);
+		if (stock == GTK_STOCK_GOTO_LAST)
+			gtk_widget_set_usize (img, 10, 6);
+		gtk_container_add (GTK_CONTAINER (bbox), img);
+		gtk_widget_show (img);
 		gtk_box_pack_start (GTK_BOX (box), wid, 0, 0, 0);
 	}
 
+	g_signal_connect (G_OBJECT (wid), "clicked",
+							G_CALLBACK (callback), userdata);
 	gtk_widget_show (wid);
 	if (tip)
 		add_tip (wid, tip);
@@ -544,10 +485,36 @@ gtkutil_clist_selection (GtkWidget * clist)
 	return -1;
 }
 
+static int
+int_compare (const int * elem1, const int * elem2)
+{
+	return (*elem1) - (*elem2);
+}
+
+int
+gtkutil_clist_multiple_selection (GtkWidget * clist, int ** rows, const int max_rows)
+{
+	int i = 0;
+	GList *tmp_clist;
+	*rows = malloc (sizeof (int) * max_rows );
+	memset( *rows, -1, max_rows * sizeof(int) );
+
+	for( tmp_clist = GTK_CLIST(clist)->selection;
+			tmp_clist && i < max_rows; tmp_clist = tmp_clist->next, i++)
+	{
+		(*rows)[i] = GPOINTER_TO_INT( tmp_clist->data );
+	}
+	qsort(*rows, i, sizeof(int), (void *)int_compare);
+	return i;
+
+}
+
 void
 add_tip (GtkWidget * wid, char *text)
 {
-	GtkTooltips *tip = gtk_tooltips_new ();
+	static GtkTooltips *tip = NULL;
+	if (!tip)
+		tip = gtk_tooltips_new ();
 	gtk_tooltips_set_tip (tip, wid, text, 0);
 }
 
@@ -573,6 +540,9 @@ gtkutil_window_new (char *title, char *role, int width, int height, int flags)
 
 	win = gtk_window_new (GTK_WINDOW_TOPLEVEL);
 	gtkutil_set_icon (win);
+#ifdef WIN32
+	gtk_window_set_wmclass (GTK_WINDOW (win), "XChat", "xchat");
+#endif
 	gtk_window_set_title (GTK_WINDOW (win), title);
 	gtk_window_set_default_size (GTK_WINDOW (win), width, height);
 	gtk_window_set_role (GTK_WINDOW (win), role);
