@@ -34,6 +34,7 @@
 #include <gtk/gtklabel.h>
 #include <gtk/gtkmenuitem.h>
 #include <gtk/gtkprogressbar.h>
+#include <gtk/gtkscrolledwindow.h>
 #include <gtk/gtkstock.h>
 #include <gtk/gtktable.h>
 #include <gtk/gtknotebook.h>
@@ -63,6 +64,15 @@
 #include "pixmaps.h"
 #include "xtext.h"
 
+#ifdef USE_GTKSPELL
+#include <gtk/gtktextview.h>
+#include <gtkspell/gtkspell.h>
+#endif
+
+#ifdef USE_LIBSEXY
+#include "sexy-spell-entry.h"
+#endif
+
 #define GUI_SPACING (3)
 #define GUI_BORDER (1)
 #define SCROLLBAR_SPACING (2)
@@ -91,8 +101,60 @@ static PangoAttrList *newmsg_list;
 static PangoAttrList *plain_list = NULL;
 
 
+#ifdef USE_GTKSPELL
+
+/* use these when it's a GtkTextView instead of GtkEntry */
+
+char *
+SPELL_ENTRY_GET_TEXT (GtkWidget *entry)
+{
+	static char *last = NULL;	/* warning: don't overlap 2 GET_TEXT calls! */
+	GtkTextBuffer *buf = gtk_text_view_get_buffer (GTK_TEXT_VIEW (entry));
+	GtkTextIter start_iter, end_iter;
+
+	gtk_text_buffer_get_iter_at_offset (buf, &start_iter, 0);
+	gtk_text_buffer_get_end_iter (buf, &end_iter);
+	g_free (last);
+	last = gtk_text_buffer_get_text (buf, &start_iter, &end_iter, FALSE);
+	return last;
+}
+
+void
+SPELL_ENTRY_SET_POS (GtkWidget *entry, int pos)
+{
+	GtkTextIter iter;
+	GtkTextBuffer *buf = gtk_text_view_get_buffer (GTK_TEXT_VIEW (entry));
+
+	gtk_text_buffer_get_iter_at_offset (buf, &iter, pos);
+	gtk_text_buffer_place_cursor (buf, &iter);
+}
+
+int
+SPELL_ENTRY_GET_POS (GtkWidget *entry)
+{
+	GtkTextIter cursor;
+	GtkTextBuffer *buf = gtk_text_view_get_buffer (GTK_TEXT_VIEW (entry));
+
+	gtk_text_buffer_get_iter_at_mark (buf, &cursor, gtk_text_buffer_get_insert (buf));
+	return gtk_text_iter_get_offset (&cursor);
+}
+
+void
+SPELL_ENTRY_INSERT (GtkWidget *entry, const char *text, int len, int *pos)
+{
+	GtkTextIter iter;
+	GtkTextBuffer *buf = gtk_text_view_get_buffer (GTK_TEXT_VIEW (entry));
+
+	/* len is bytes. pos is chars. */
+	gtk_text_buffer_get_iter_at_offset (buf, &iter, *pos);
+	gtk_text_buffer_insert (buf, &iter, text, len);
+	*pos += g_utf8_strlen (text, len);
+}
+
+#endif
+
 static PangoAttrList *
-mg_attr_list_create (GdkColor *col, gboolean small)
+mg_attr_list_create (GdkColor *col, int size)
 {
 	PangoAttribute *attr;
 	PangoAttrList *list;
@@ -107,9 +169,9 @@ mg_attr_list_create (GdkColor *col, gboolean small)
 		pango_attr_list_insert (list, attr);
 	}
 
-	if (small)
+	if (size > 0)
 	{
-		attr = pango_attr_scale_new (PANGO_SCALE_SMALL);
+		attr = pango_attr_scale_new (size == 1 ? PANGO_SCALE_SMALL : PANGO_SCALE_X_SMALL);
 		attr->start_index = 0;
 		attr->end_index = 0xffff;
 		pango_attr_list_insert (list, attr);
@@ -207,10 +269,21 @@ unflash_window (GtkWidget *win)
 #endif
 #endif
 
+/* flash the taskbar button */
+
+void
+fe_flash_window (session *sess)
+{
+#if defined(WIN32) || defined(USE_XLIB)
+	if (fe_gui_info (sess, 0) != 1)	/* only do it if not focused */
+		flash_window (sess->gui->window);
+#endif
+}
+
 /* set a tab plain, red, light-red, or blue */
 
 void
-fe_set_tab_color (struct session *sess, int col, int flash)
+fe_set_tab_color (struct session *sess, int col)
 {
 	struct session *server_sess = sess->server->server_session;
 	if (sess->gui->is_tab && (col == 0 || sess != current_tab))
@@ -270,11 +343,6 @@ fe_set_tab_color (struct session *sess, int col, int flash)
 			break;
 		}
 	}
-
-#if defined(WIN32) || defined(USE_XLIB)
-	if (flash && prefs.flash_hilight && fe_gui_info (sess, 0) != 1)
-		flash_window (sess->gui->window);
-#endif
 }
 
 static void
@@ -331,10 +399,10 @@ mg_inputbox_focus (GtkWidget *widget, GdkEventFocus *event, session_gui *gui)
 	return FALSE;
 }
 
-static void
+void
 mg_inputbox_cb (GtkWidget *igad, session_gui *gui)
 {
-	char *cmd = GTK_ENTRY (igad)->text;
+	char *cmd;
 	static int ignore = FALSE;
 	GSList *list;
 	session *sess = NULL;
@@ -342,6 +410,7 @@ mg_inputbox_cb (GtkWidget *igad, session_gui *gui)
 	if (ignore)
 		return;
 
+	cmd = SPELL_ENTRY_GET_TEXT (igad);
 	if (cmd[0] == 0)
 		return;
 
@@ -349,7 +418,7 @@ mg_inputbox_cb (GtkWidget *igad, session_gui *gui)
 
 	/* avoid recursive loop */
 	ignore = TRUE;
-	gtk_entry_set_text (GTK_ENTRY (igad), "");
+	SPELL_ENTRY_SET_TEXT (igad, "");
 	ignore = FALSE;
 
 	/* where did this event come from? */
@@ -372,13 +441,14 @@ mg_inputbox_cb (GtkWidget *igad, session_gui *gui)
 
 	if (sess)
 		handle_multiline (sess, cmd, TRUE, FALSE);
+
 	free (cmd);
 }
 
 void
 fe_set_title (session *sess)
 {
-	char tbuf[256];
+	char tbuf[384];
 	int type;
 
 	if (sess->gui->is_tab && sess != current_tab)
@@ -495,9 +565,9 @@ mg_focus (session *sess)
 	current_sess = sess;
 
 	/* dirty trick to avoid auto-selection */
-	GTK_ENTRY (sess->gui->input_box)->editable = 0;
+	SPELL_ENTRY_SET_EDITABLE (sess->gui->input_box, FALSE);
 	gtk_widget_grab_focus (sess->gui->input_box);
-	gtk_editable_set_editable (GTK_EDITABLE (sess->gui->input_box), TRUE);
+	SPELL_ENTRY_SET_EDITABLE (sess->gui->input_box, TRUE);
 
 	sess->server->front_session = sess;
 
@@ -518,7 +588,7 @@ mg_focus (session *sess)
 		/* when called via mg_changui_new, is_tab might be true, but
 			sess->res->tab is still NULL. */
 		if (sess->res->tab)
-			fe_set_tab_color (sess, 0, FALSE);
+			fe_set_tab_color (sess, 0);
 	}
 }
 
@@ -587,7 +657,7 @@ mg_unpopulate (session *sess)
 		prefs.paned_pos = gui->pane_pos = i;
 	}
 
-	res->input_text = strdup (GTK_ENTRY (gui->input_box)->text);
+	res->input_text = strdup (SPELL_ENTRY_GET_TEXT (gui->input_box));
 	res->topic_text = strdup (GTK_ENTRY (gui->topic_entry)->text);
 	res->limit_text = strdup (GTK_ENTRY (gui->limit_entry)->text);
 	res->key_text = strdup (GTK_ENTRY (gui->key_entry)->text);
@@ -646,19 +716,35 @@ mg_restore_entry (GtkWidget *entry, char **text)
 	gtk_editable_set_position (GTK_EDITABLE (entry), -1);
 }
 
+static void
+mg_restore_speller (GtkWidget *entry, char **text)
+{
+	if (*text)
+	{
+		SPELL_ENTRY_SET_TEXT (entry, *text);
+		free (*text);
+		*text = NULL;
+	} else
+	{
+		SPELL_ENTRY_SET_TEXT (entry, "");
+	}
+	SPELL_ENTRY_SET_POS (entry, -1);
+}
+
 void
 mg_set_topic_tip (session *sess)
 {
-	char buf[512];
+	char *text;
 
 	switch (sess->type)
 	{
 	case SESS_CHANNEL:
 		if (sess->topic)
 		{
-			snprintf (buf, sizeof (buf), _("Topic for %s is: %s"), sess->channel,
+			text = g_strdup_printf (_("Topic for %s is: %s"), sess->channel,
 						 sess->topic);
-			add_tip (sess->gui->topic_entry, buf);
+			add_tip (sess->gui->topic_entry, text);
+			g_free (text);
 		} else
 			add_tip (sess->gui->topic_entry, _("No topic is set"));
 		break;
@@ -769,7 +855,7 @@ mg_populate (session *sess)
 
 	/* restore all the GtkEntry's */
 	mg_restore_entry (gui->topic_entry, &res->topic_text);
-	mg_restore_entry (gui->input_box, &res->input_text);
+	mg_restore_speller (gui->input_box, &res->input_text);
 	mg_restore_entry (gui->key_entry, &res->key_text);
 	mg_restore_entry (gui->limit_entry, &res->limit_text);
 	mg_restore_label (gui->laginfo, &res->lag_text);
@@ -925,6 +1011,9 @@ mg_tab_close_cb (GtkWidget *dialog, gint arg1, session *sess)
 	gtk_widget_destroy (dialog);
 	if (arg1 == GTK_RESPONSE_OK && is_session (sess))
 	{
+		/* force it NOT to send individual PARTs */
+		sess->server->sent_quit = TRUE;
+
 		for (list = sess_list; list;)
 		{
 			next = list->next;
@@ -933,6 +1022,9 @@ mg_tab_close_cb (GtkWidget *dialog, gint arg1, session *sess)
 				fe_close_window ((session *)list->data);
 			list = next;
 		}
+
+		/* just send one QUIT - better for BNCs */
+		sess->server->sent_quit = FALSE;
 		fe_close_window (sess);
 	}
 }
@@ -1293,7 +1385,7 @@ mg_tab_contextmenu_cb (chanview *cv, chan *ch, int tag, gpointer ud, GdkEventBut
 	session *sess = ud;
 
 	/* shift-click to close a tab */
-	if (event->state & GDK_SHIFT_MASK)
+	if ((event->state & GDK_SHIFT_MASK) && event->type == GDK_BUTTON_PRESS)
 	{
 		mg_xbutton_cb (cv, ch, tag, ud);
 		return FALSE;
@@ -1372,7 +1464,7 @@ mg_dnd_drop_file (session *sess, char *target, char *uri)
 			if (fname)
 			{
 				/* dcc_send() expects utf-8 */
-				p = g_filename_to_utf8 (fname, -1, 0, 0, 0);
+				p = xchat_filename_to_utf8 (fname, -1, 0, 0, 0);
 				if (p)
 				{
 					dcc_send (sess, target, p, prefs.dcc_max_send_cps, 0);
@@ -1679,9 +1771,24 @@ mg_link_irctab (session *sess, int focus)
 }
 
 void
-mg_link_cb (GtkWidget *but, gpointer userdata)
+mg_detach (session *sess, int mode)
 {
-	mg_link_irctab (current_sess, 1);
+	switch (mode)
+	{
+	/* detach only */
+	case 1:
+		if (sess->gui->is_tab)
+			mg_link_irctab (sess, 1);
+		break;
+	/* attach only */
+	case 2:
+		if (!sess->gui->is_tab)
+			mg_link_irctab (sess, 1);
+		break;
+	/* toggle */
+	default:
+		mg_link_irctab (sess, 1);
+	}
 }
 
 static int
@@ -1919,14 +2026,16 @@ mg_dialog_button_cb (GtkWidget *wid, char *cmd)
 	if (topic)
 		host = topic + 1;
 
-	auto_insert (buf, sizeof (buf), cmd, 0, 0, "", "", "", host, "", current_sess->channel);
+	auto_insert (buf, sizeof (buf), cmd, 0, 0, "", "", "",
+					 server_get_network (current_sess->server, TRUE), host, "",
+					 current_sess->channel);
 
 	handle_command (current_sess, buf, TRUE);
 
 	/* dirty trick to avoid auto-selection */
-	GTK_ENTRY (current_sess->gui->input_box)->editable = 0;
+	SPELL_ENTRY_SET_EDITABLE (current_sess->gui->input_box, FALSE);
 	gtk_widget_grab_focus (current_sess->gui->input_box);
-	gtk_editable_set_editable (GTK_EDITABLE (current_sess->gui->input_box), TRUE);
+	SPELL_ENTRY_SET_EDITABLE (current_sess->gui->input_box, TRUE);
 }
 
 static void
@@ -2028,7 +2137,7 @@ mg_word_clicked (GtkWidget *xtext, char *word, GdkEventButton *even)
 			return;
 		}
 
-		if (even->state & GDK_CONTROL_MASK)
+		if ((even->state & 13) == prefs.gui_url_mod)
 		{
 			switch (mg_word_check (xtext, word, strlen (word)))
 			{
@@ -2436,7 +2545,7 @@ mg_inputbox_rightclick (GtkEntry *entry, GtkWidget *menu)
 static void
 mg_create_entry (session *sess, GtkWidget *box)
 {
-	GtkWidget *hbox, *but, *entry;
+	GtkWidget *sw, *hbox, *but, *entry;
 	session_gui *gui = sess->gui;
 
 	hbox = gtk_hbox_new (FALSE, 0);
@@ -2452,18 +2561,42 @@ mg_create_entry (session *sess, GtkWidget *box)
 	g_signal_connect (G_OBJECT (but), "clicked",
 							G_CALLBACK (mg_nickclick_cb), NULL);
 
+#ifdef USE_GTKSPELL
+	gui->input_box = entry = gtk_text_view_new ();
+	gtk_widget_set_size_request (entry, 0, 1);
+	gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW (entry), GTK_WRAP_NONE);
+	gtk_text_view_set_accepts_tab (GTK_TEXT_VIEW (entry), FALSE);
+	if (prefs.gui_input_spell)
+		gtkspell_new_attach (GTK_TEXT_VIEW (entry), NULL, NULL);
+
+	sw = gtk_scrolled_window_new (NULL, NULL);
+	gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (sw),
+													 GTK_SHADOW_IN);
+	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sw),
+												GTK_POLICY_NEVER,
+												GTK_POLICY_NEVER);
+	gtk_container_add (GTK_CONTAINER (sw), entry);
+	gtk_container_add (GTK_CONTAINER (hbox), sw);
+#else
+#ifdef USE_LIBSEXY
+	gui->input_box = entry = sexy_spell_entry_new ();
+	sexy_spell_entry_set_checked ((SexySpellEntry *)entry, prefs.gui_input_spell);
+#else
 	gui->input_box = entry = gtk_entry_new ();
-	gtk_widget_set_name (entry, "xchat-inputbox");
+#endif
 	gtk_entry_set_max_length (GTK_ENTRY (gui->input_box), 2048);
+	g_signal_connect (G_OBJECT (entry), "activate",
+							G_CALLBACK (mg_inputbox_cb), gui);
+	gtk_container_add (GTK_CONTAINER (hbox), entry);
+#endif
+
+	gtk_widget_set_name (entry, "xchat-inputbox");
 	g_signal_connect (G_OBJECT (entry), "key_press_event",
 							G_CALLBACK (key_handle_key_press), NULL);
 	g_signal_connect (G_OBJECT (entry), "focus_in_event",
 							G_CALLBACK (mg_inputbox_focus), gui);
-	g_signal_connect (G_OBJECT (entry), "activate",
-							G_CALLBACK (mg_inputbox_cb), gui);
 	g_signal_connect (G_OBJECT (entry), "populate_popup",
 							G_CALLBACK (mg_inputbox_rightclick), NULL);
-	gtk_container_add (GTK_CONTAINER (hbox), entry);
 	gtk_widget_grab_focus (entry);
 
 	if (prefs.style_inputbox)
@@ -2940,6 +3073,13 @@ fe_set_away (server *serv)
 	}
 }
 
+void
+fe_set_color_paste (session *sess, int status)
+{
+	sess->color_paste = status;
+	if (!sess->gui->is_tab || sess == current_tab)
+		GTK_XTEXT (sess->gui->xtext)->color_paste = status;
+}
 void
 fe_set_channel (session *sess)
 {
@@ -3101,6 +3241,9 @@ fe_server_callback (server *serv)
 void
 fe_session_callback (session *sess)
 {
+	if (sess->res->banlist_window)
+		mg_close_gen (NULL, sess->res->banlist_window);
+
 	if (sess->res->input_text)
 		free (sess->res->input_text);
 
