@@ -99,6 +99,10 @@ struct mymenu
 	guint key;				/* GDK_x */
 };
 
+#define XCMENU_DOLIST 1
+#define XCMENU_SHADED 1
+#define XCMENU_MARKUP 2
+#define XCMENU_MNEMONIC 4
 
 /* execute a userlistbutton/popupmenu command */
 
@@ -138,8 +142,9 @@ nick_command_parse (session *sess, char *cmd, char *nick, char *allnick)
 	len = strlen (cmd) + strlen (nick) + strlen (allnick) + 512;
 	buf = malloc (len);
 
-	auto_insert (buf, len, cmd, 0, 0, allnick, sess->channel, "", host,
-						sess->server->nick, nick);
+	auto_insert (buf, len, cmd, 0, 0, allnick, sess->channel, "",
+					 server_get_network (sess->server, TRUE), host,
+					 sess->server->nick, nick);
 
 	nick_command (sess, buf);
 
@@ -265,13 +270,13 @@ menu_quick_item (char *cmd, char *label, GtkWidget * menu, int flags,
 		item = gtk_menu_item_new ();
 	else
 	{
-		if (flags & (1 << 1))
+		if (flags & XCMENU_MARKUP)
 		{
 			item = gtk_menu_item_new_with_label ("");
 			gtk_label_set_markup (GTK_LABEL (GTK_BIN (item)->child), label);
 		} else
 		{
-			if (flags & (1 << 2))
+			if (flags & XCMENU_MNEMONIC)
 				item = gtk_menu_item_new_with_mnemonic (label);
 			else
 				item = gtk_menu_item_new_with_label (label);
@@ -282,7 +287,7 @@ menu_quick_item (char *cmd, char *label, GtkWidget * menu, int flags,
 	if (cmd)
 		g_signal_connect (G_OBJECT (item), "activate",
 								G_CALLBACK (popup_menu_cb), cmd);
-	if (flags & (1 << 0))
+	if (flags & XCMENU_SHADED)
 		gtk_widget_set_sensitive (GTK_WIDGET (item), FALSE);
 	gtk_widget_show (item);
 
@@ -313,10 +318,18 @@ menu_quick_sub (char *name, GtkWidget *menu, GtkWidget **sub_item_ret, int flags
 
 	/* Code to add a submenu */
 	sub_menu = gtk_menu_new ();
-	if (flags & 4)
-		sub_item = gtk_menu_item_new_with_mnemonic (name);
+	if (flags & XCMENU_MARKUP)
+	{
+		sub_item = gtk_menu_item_new_with_label ("");
+		gtk_label_set_markup (GTK_LABEL (GTK_BIN (sub_item)->child), name);
+	}
 	else
-		sub_item = gtk_menu_item_new_with_label (name);
+	{
+		if (flags & XCMENU_MNEMONIC)
+			sub_item = gtk_menu_item_new_with_mnemonic (name);
+		else
+			sub_item = gtk_menu_item_new_with_label (name);
+	}
 	gtk_menu_shell_insert (GTK_MENU_SHELL (menu), sub_item, pos);
 	gtk_widget_show (sub_item);
 	gtk_menu_item_set_submenu (GTK_MENU_ITEM (sub_item), sub_menu);
@@ -324,7 +337,7 @@ menu_quick_sub (char *name, GtkWidget *menu, GtkWidget **sub_item_ret, int flags
 	if (sub_item_ret)
 		*sub_item_ret = sub_item;
 
-	if (flags & 1)
+	if (flags & XCMENU_DOLIST)
 		/* We create a new element in the list */
 		submenu_list = g_slist_prepend (submenu_list, sub_menu);
 	return sub_menu;
@@ -402,7 +415,7 @@ menu_create (GtkWidget *menu, GSList *list, char *target, int check_path)
 		if (!strncasecmp (pop->name, "SUB", 3))
 		{
 			childcount = 0;
-			tempmenu = menu_quick_sub (pop->cmd, tempmenu, &subitem, 1, -1);
+			tempmenu = menu_quick_sub (pop->cmd, tempmenu, &subitem, XCMENU_DOLIST, -1);
 
 		} else if (!strncasecmp (pop->name, "TOGGLE", 6))
 		{
@@ -423,7 +436,7 @@ menu_create (GtkWidget *menu, GSList *list, char *target, int check_path)
 
 		} else if (!strncasecmp (pop->name, "SEP", 3))
 		{
-			menu_quick_item (0, 0, tempmenu, 1, 0);
+			menu_quick_item (0, 0, tempmenu, XCMENU_SHADED, 0);
 
 		} else
 		{
@@ -476,8 +489,8 @@ static char *str_copy = 0;		/* for all pop-up menus */
 void
 menu_nickmenu (session *sess, GdkEventButton *event, char *nick, int num_sel)
 {
-	char buf[256];
-	char *fmt;
+	char buf[512];
+	char *real, *fmt;
 	struct User *user;
 	struct away_msg *away;
 	GtkWidget *submenu, *menu = gtk_menu_new ();
@@ -493,7 +506,7 @@ menu_nickmenu (session *sess, GdkEventButton *event, char *nick, int num_sel)
 	{
 		snprintf (buf, sizeof (buf), "%d nicks selected.", num_sel);
 		menu_quick_item (0, buf, menu, 0, 0);
-		menu_quick_item (0, 0, menu, 1, 0);
+		menu_quick_item (0, 0, menu, XCMENU_SHADED, 0);
 	} else
 	{
 		user = userlist_find (sess, nick);	/* lasttalk is channel specific */
@@ -501,26 +514,33 @@ menu_nickmenu (session *sess, GdkEventButton *event, char *nick, int num_sel)
 			user = userlist_find_global (current_sess->server, nick);
 		if (user)
 		{
-			submenu = menu_quick_sub (nick, menu, NULL, 1, -1);
+			submenu = menu_quick_sub (nick, menu, NULL, XCMENU_DOLIST, -1);
 
 			/* let the translators tweak this if need be */
 			fmt = _("<tt><b>%-11s</b></tt> %s");
 
-			snprintf (buf, sizeof (buf), fmt, _("Real Name:"),
-						user->realname ? user->realname : _("Unknown"));
-			menu_quick_item (0, buf, submenu, 2, 0);
+			if (user->realname)
+			{
+				real = g_markup_escape_text (user->realname, -1);
+				snprintf (buf, sizeof (buf), fmt, _("Real Name:"), real);
+				g_free (real);
+			} else
+			{
+				snprintf (buf, sizeof (buf), fmt, _("Real Name:"), _("Unknown"));
+			}
+			menu_quick_item (0, buf, submenu, XCMENU_MARKUP, 0);
 
 			snprintf (buf, sizeof (buf), fmt, _("User:"),
 						user->hostname ? user->hostname : _("Unknown"));
-			menu_quick_item (0, buf, submenu, 2, 0);
+			menu_quick_item (0, buf, submenu, XCMENU_MARKUP, 0);
 
 			snprintf (buf, sizeof (buf), fmt, _("Country:"),
 						user->hostname ? country(user->hostname) : _("Unknown"));
-			menu_quick_item (0, buf, submenu, 2, 0);
+			menu_quick_item (0, buf, submenu, XCMENU_MARKUP, 0);
 
 			snprintf (buf, sizeof (buf), fmt, _("Server:"),
 						user->servername ? user->servername : _("Unknown"));
-			menu_quick_item (0, buf, submenu, 2, 0);
+			menu_quick_item (0, buf, submenu, XCMENU_MARKUP, 0);
 
 			if (user->away)
 			{
@@ -528,9 +548,11 @@ menu_nickmenu (session *sess, GdkEventButton *event, char *nick, int num_sel)
 				if (away)
 				{
 					char *msg = strip_color (away->message ? away->message : _("Unknown"), -1, STRIP_ALL);
-					snprintf (buf, sizeof (buf), fmt, _("Away Msg:"), msg);
-					menu_quick_item (0, buf, submenu, 2, 0);
+					real = g_markup_escape_text (msg, -1);
 					free (msg);
+					snprintf (buf, sizeof (buf), fmt, _("Away Msg:"), real);
+					g_free (real);
+					menu_quick_item (0, buf, submenu, XCMENU_MARKUP, 0);
 				}
 			}
 
@@ -546,10 +568,10 @@ menu_nickmenu (session *sess, GdkEventButton *event, char *nick, int num_sel)
 				snprintf (buf, sizeof (buf), fmt, _("Last Msg:"), _("Unknown"));
 			}
 
-			menu_quick_item (0, buf, submenu, 2, 0);
+			menu_quick_item (0, buf, submenu, XCMENU_MARKUP, 0);
 
 			menu_quick_endsub ();
-			menu_quick_item (0, 0, menu, 1, 0);
+			menu_quick_item (0, 0, menu, XCMENU_SHADED, 0);
 		}
 	}
 
@@ -717,13 +739,13 @@ menu_urlmenu (GdkEventButton *event, char *url)
 		chop = g_utf8_offset_to_pointer (tmp, 48);
 		chop[0] = chop[1] = chop[2] = '.';
 		chop[3] = 0;
-		menu_quick_item (0, tmp, menu, 1, 0);
+		menu_quick_item (0, tmp, menu, XCMENU_SHADED, 0);
 		free (tmp);
 	} else
 	{
-		menu_quick_item (0, str_copy, menu, 1, 0);
+		menu_quick_item (0, str_copy, menu, XCMENU_SHADED, 0);
 	}
-	menu_quick_item (0, 0, menu, 1, 0);
+	menu_quick_item (0, 0, menu, XCMENU_SHADED, 0);
 
 	/* Two hardcoded entries */
 	if (strncmp (str_copy, "irc://", 6) == 0 ||
@@ -783,8 +805,8 @@ menu_chanmenu (struct session *sess, GdkEventButton * event, char *chan)
 
 	menu = gtk_menu_new ();
 
-	menu_quick_item (0, chan, menu, 1, str_copy);
-	menu_quick_item (0, 0, menu, 1, str_copy);
+	menu_quick_item (0, chan, menu, XCMENU_SHADED, str_copy);
+	menu_quick_item (0, 0, menu, XCMENU_SHADED, str_copy);
 
 	if (!is_joined)
 		menu_quick_item_with_callback (menu_chan_join, _("Join Channel"), menu,
@@ -824,7 +846,7 @@ static void
 usermenu_create (GtkWidget *menu)
 {
 	menu_create (menu, usermenu_list, "", FALSE);
-	menu_quick_item (0, 0, menu, 1, 0);	/* sep */
+	menu_quick_item (0, 0, menu, XCMENU_SHADED, 0);	/* sep */
 	menu_quick_item_with_callback (menu_usermenu, _("Edit This Menu..."), menu, 0);
 }
 
@@ -922,28 +944,10 @@ menu_rawlog (GtkWidget * wid, gpointer none)
 	open_rawlog (current_sess->server);
 }
 
-static void
-menu_autodccsend (GtkWidget * wid, gpointer none)
-{
-	prefs.autodccsend = !prefs.autodccsend;
-#ifndef WIN32
-	if (prefs.autodccsend)
-	{
-		if (!strcmp ((char *)g_get_home_dir (), prefs.dccdir))
-		{
-			fe_message (_("*WARNING*\n"
-							 "Auto accepting DCC to your home directory\n"
-							 "can be dangerous and is exploitable. Eg:\n"
-							 "Someone could send you a .bash_profile"), FE_MSG_WARN);
-		}
-	}
-#endif
-}
-
 static void
 menu_detach (GtkWidget * wid, gpointer none)
 {
-	mg_link_cb (NULL, NULL);
+	mg_detach (current_sess, 0);
 }
 
 static void
@@ -993,42 +997,6 @@ menu_savebuffer (GtkWidget * wid, gpointer none)
 							current_sess, NULL, FRF_WRITE);
 }
 
-static void
-menu_wallops (GtkWidget * wid, gpointer none)
-{
-	char mode[3];
-	server *serv = current_sess->server;
-	prefs.wallops = !prefs.wallops;
-	if (serv->connected)
-	{
-		mode[1] = 'w';
-		mode[2] = '\0';
-		if (prefs.wallops)
-			mode[0] = '+';
-		else
-			mode[0] = '-';
-		serv->p_mode (serv, serv->nick, mode);
-	}
-}
-
-static void
-menu_servernotice (GtkWidget * wid, gpointer none)
-{
-	char mode[3];
-	server *serv = current_sess->server;
-	prefs.servernotice = !prefs.servernotice;
-	if (serv->connected)
-	{
-		mode[1] = 's';
-		mode[2] = '\0';
-		if (prefs.servernotice)
-			mode[0] = '+';
-		else
-			mode[0] = '-';
-		serv->p_mode (serv, serv->nick, mode);
-	}
-}
-
 static void
 menu_disconnect (GtkWidget * wid, gpointer none)
 {
@@ -1083,6 +1051,7 @@ menu_join (GtkWidget * wid, gpointer none)
 	hbox = gtk_hbox_new (TRUE, 0);
 
 	entry = gtk_entry_new ();
+	GTK_ENTRY (entry)->editable = 0;	/* avoid auto-selection */
 	gtk_entry_set_text (GTK_ENTRY (entry), "#");
 	g_signal_connect (G_OBJECT (entry), "activate",
 						 	G_CALLBACK (menu_join_entry_cb), dialog);
@@ -1097,6 +1066,9 @@ menu_join (GtkWidget * wid, gpointer none)
 	gtk_container_add (GTK_CONTAINER (GTK_DIALOG (dialog)->vbox), hbox);
 
 	gtk_widget_show_all (dialog);
+
+	gtk_editable_set_editable (GTK_EDITABLE (entry), TRUE);
+	gtk_editable_set_position (GTK_EDITABLE (entry), 1);
 }
 
 static void
@@ -1105,34 +1077,6 @@ menu_away (GtkWidget * wid, gpointer none)
 	handle_command (current_sess, "away", FALSE);
 }
 
-static void
-menu_invisible (GtkWidget * wid, gpointer none)
-{
-	char mode[3];
-	server *serv = current_sess->server;
-	prefs.invisible = !prefs.invisible;
-	if (current_sess->server->connected)
-	{
-		mode[1] = 'i';
-		mode[2] = '\0';
-		if (prefs.invisible)
-			mode[0] = '+';
-		else
-			mode[0] = '-';
-		serv->p_mode (serv, serv->nick, mode);
-	}
-}
-
-#if 0
-static void
-menu_savedefault (GtkWidget * wid, gpointer none)
-{
-	palette_save ();
-	if (save_config ())
-		fe_message (_("Settings saved."), FE_MSG_INFO);
-}
-#endif
-
 static void
 menu_chanlist (GtkWidget * wid, gpointer none)
 {
@@ -1168,6 +1112,7 @@ menu_pluginlist (void)
 
 #define usercommands_help  _("User Commands - Special codes:\n\n"\
                            "%c  =  current channel\n"\
+									"%e  =  current network name\n"\
 									"%m  =  machine info\n"\
                            "%n  =  your nick\n"\
 									"%t  =  time/date\n"\
@@ -1184,6 +1129,7 @@ menu_pluginlist (void)
 #define ulbutton_help       _("Userlist Buttons - Special codes:\n\n"\
                            "%a  =  all selected nicks\n"\
                            "%c  =  current channel\n"\
+									"%e  =  current network name\n"\
                            "%h  =  selected nick's hostname\n"\
 									"%m  =  machine info\n"\
                            "%n  =  your nick\n"\
@@ -1193,6 +1139,7 @@ menu_pluginlist (void)
 #define dlgbutton_help      _("Dialog Buttons - Special codes:\n\n"\
                            "%a  =  all selected nicks\n"\
                            "%c  =  current channel\n"\
+									"%e  =  current network name\n"\
                            "%h  =  selected nick's hostname\n"\
 									"%m  =  machine info\n"\
                            "%n  =  your nick\n"\
@@ -1201,6 +1148,7 @@ menu_pluginlist (void)
 
 #define ctcp_help          _("CTCP Replies - Special codes:\n\n"\
                            "%d  =  data (the whole ctcp)\n"\
+									"%e  =  current network name\n"\
 									"%m  =  machine info\n"\
                            "%s  =  nick who sent the ctcp\n"\
                            "%t  =  time/date\n"\
@@ -1276,59 +1224,6 @@ menu_ctcpguiopen (void)
 							 "ctcpreply.conf", ctcp_help);
 }
 
-#if 0
-static void
-menu_reload (void)
-{
-	char *buf = malloc (strlen (default_file ()) + 12);
-	load_config ();
-	sprintf (buf, "%s reloaded.", default_file ());
-	fe_message (buf, FE_MSG_INFO);
-	free (buf);
-}
-#endif
-
-static void
-menu_autorejoin (GtkWidget *wid, gpointer none)
-{
-	prefs.autorejoin = !prefs.autorejoin;
-}
-
-static void
-menu_autoreconnect (GtkWidget *wid, gpointer none)
-{
-	prefs.autoreconnect = !prefs.autoreconnect;
-}
-
-static void
-menu_autoreconnectonfail (GtkWidget *wid, gpointer none)
-{
-	prefs.autoreconnectonfail = !prefs.autoreconnectonfail;
-}
-
-static void
-menu_autodialog (GtkWidget *wid, gpointer none)
-{
-	if (GTK_CHECK_MENU_ITEM (wid)->active)
-		prefs.autodialog = 1;
-	else
-		prefs.autodialog = 0;
-}
-
-static void
-menu_autodccchat (GtkWidget *wid, gpointer none)
-{
-	prefs.autodccchat = !prefs.autodccchat;
-}
-
-#if 0
-static void
-menu_saveexit (GtkWidget *wid, gpointer none)
-{
-	prefs.autosave = !prefs.autosave;
-}
-#endif
-
 static void
 menu_docs (GtkWidget *wid, gpointer none)
 {
@@ -1424,7 +1319,7 @@ menu_metres_both (GtkWidget *item, gpointer none)
 
 static struct mymenu mymenu[] = {
 	{N_("_XChat"), 0, 0, M_NEWMENU, 0, 0, 1},
-	{N_("_Server List..."), menu_open_server_list, (char *)&pix_book, M_MENUPIX, 0, 0, 1, GDK_s},
+	{N_("Network Li_st..."), menu_open_server_list, (char *)&pix_book, M_MENUPIX, 0, 0, 1, GDK_s},
 	{0, 0, 0, M_SEP, 0, 0, 0},
 
 	{N_("_New"), 0, GTK_STOCK_NEW, M_MENUSUB, 0, 0, 1},
@@ -1471,7 +1366,7 @@ static struct mymenu mymenu[] = {
 	{N_("_Server"), 0, 0, M_NEWMENU, 0, 0, 1},
 	{N_("_Disconnect"), menu_disconnect, GTK_STOCK_DISCONNECT, M_MENUSTOCK, MENU_ID_DISCONNECT, 0, 1},
 	{N_("_Reconnect"), menu_reconnect, GTK_STOCK_CONNECT, M_MENUSTOCK, MENU_ID_RECONNECT, 0, 1},
-	{N_("Join Channel"), menu_join, GTK_STOCK_JUMP_TO, M_MENUSTOCK, MENU_ID_JOIN, 0, 1},
+	{N_("Join Channel..."), menu_join, GTK_STOCK_JUMP_TO, M_MENUSTOCK, MENU_ID_JOIN, 0, 1},
 	{0, 0, 0, M_SEP, 0, 0, 0},
 #define AWAY_OFFSET (37)
 	{N_("Marked Away"), menu_away, 0, M_MENUTOG, MENU_ID_AWAY, 0, 1, GDK_a},
@@ -1479,7 +1374,7 @@ static struct mymenu mymenu[] = {
 	{N_("_Usermenu"), 0, 0, M_NEWMENU, MENU_ID_USERMENU, 0, 1},	/* 38 */
 
 	{N_("S_ettings"), 0, 0, M_NEWMENU, 0, 0, 1},
-	{N_("Preferences..."), menu_settings, GTK_STOCK_PREFERENCES, M_MENUSTOCK, 0, 0, 1},
+	{N_("_Preferences"), menu_settings, GTK_STOCK_PREFERENCES, M_MENUSTOCK, 0, 0, 1},
 
 	{N_("Advanced"), 0, GTK_STOCK_JUSTIFY_LEFT, M_MENUSUB, 0, 0, 1},
 		{N_("Auto Replace..."), menu_rpopup, 0, M_MENUITEM, 0, 0, 1},
@@ -1700,7 +1595,7 @@ menu_add_item (GtkWidget *menu, menu_entry *me)
 	menu = menu_find_path (menu, me->path);
 	if (menu)
 	{
-		item = menu_quick_item (me->cmd, me->label, menu, 4, 0);
+		item = menu_quick_item (me->cmd, me->label, menu, me->markup ? XCMENU_MARKUP : XCMENU_MNEMONIC, 0);
 		if (me->pos != -1)
 			gtk_menu_reorder_child (GTK_MENU (menu), item, me->pos);
 	}
@@ -1715,7 +1610,7 @@ menu_add_sub (GtkWidget *menu, menu_entry *me)
 	if (me->path[0] != 0)
 		menu = menu_find_path (menu, me->path);
 	if (menu)
-		menu_quick_sub (me->label, menu, &item, 4, me->pos);
+		menu_quick_sub (me->label, menu, &item, me->markup ? XCMENU_MARKUP : XCMENU_MNEMONIC, me->pos);
 	return item;
 }
 
@@ -1749,10 +1644,21 @@ menu_add_cb (GtkWidget *menu, menu_entry *me)
 	}
 }
 
-void
+char *
 fe_menu_add (menu_entry *me)
 {
+	char *text;
+
 	menu_foreach_gui (me, menu_add_cb);
+
+	if (!me->markup)
+		return NULL;
+
+	if (!pango_parse_markup (me->label, -1, 0, NULL, &text, NULL, NULL))
+		return NULL;
+
+	/* return the label with markup stripped */
+	return text;
 }
 
 void
