@@ -39,24 +39,27 @@
 #include <gtk/gtktreeview.h>
 #include <gtk/gtkhbbox.h>
 #include <gtk/gtkhseparator.h>
-#include <gtk/gtkmenu.h>
-#include <gtk/gtkmenuitem.h>
 #include <gtk/gtkradiobutton.h>
-#include <gtk/gtkoptionmenu.h>
+#include <gtk/gtkcombobox.h>
 #include <gtk/gtkliststore.h>
 #include <gtk/gtktreestore.h>
 #include <gtk/gtktreeselection.h>
 #include <gtk/gtkcellrenderertext.h>
 #include <gtk/gtkhscale.h>
 #ifdef WIN32
-#include <windows.h>
 #include "../common/fe.h"
 #endif
-
+#ifdef USE_GTKSPELL
+#include <gtk/gtktextview.h>
+#include <gtkspell/gtkspell.h>
+#endif
+#ifdef USE_LIBSEXY
+#include "sexy-spell-entry.h"
+#endif
 
 GtkStyle *create_input_style (GtkStyle *);
 
-#define LABEL_INDENT 6
+#define LABEL_INDENT 12
 
 static int last_selected_page = 0;
 static int last_selected_row = 0; /* sound row */
@@ -72,7 +75,7 @@ enum
 	ST_ENTRY,
 	ST_EFONT,
 	ST_EFILE,
-	ST_EOPEN,
+	ST_EFOLDER,
 	ST_MENU,
 	ST_RADIO,
 	ST_NUMBER,
@@ -111,26 +114,39 @@ static const setting textbox_settings[] =
 	{ST_HSCALE, N_("Blue:"), P_OFFINTNL(tint_blue),0,0,0},
 
 	{ST_HEADER,	N_("Time Stamps"),0,0,0},
+	{ST_TOGGLE, N_("Enable time stamps"), P_OFFINTNL(timestamp),0,0,2},
 	{ST_ENTRY,  N_("Time stamp format:"), P_OFFSETNL(stamp_format),
 					N_("See strftime manpage for details."),0,sizeof prefs.stamp_format},
-	{ST_TOGGLE, N_("Enable time stamps"), P_OFFINTNL(timestamp),0,0,0},
 
 	{ST_END, 0, 0, 0, 0, 0}
 };
 
+static const char *const tabcompmenu[] = 
+{
+	N_("A-Z"),
+	N_("Last-spoke order"),
+	NULL
+};
+
 static const setting inputbox_settings[] =
 {
-	{ST_HEADER, N_("Nick Completion"),0,0,0},
-	{ST_TOGGLE, N_("Automatic nick completion"), P_OFFINTNL(nickcompletion),
-					N_("Completes nick names without using the TAB key"),0,0},
-	{ST_ENTRY,	N_("Nick completion suffix:"), P_OFFSETNL(nick_suffix),0,0,sizeof prefs.nick_suffix},
+	{ST_HEADER, N_("Input box"),0,0,0},
+	{ST_TOGGLE, N_("Use the Text box font and colors"), P_OFFINTNL(style_inputbox),0,0,0},
+#if defined(USE_GTKSPELL) || defined(USE_LIBSEXY)
+	{ST_TOGGLE, N_("Spell checking"), P_OFFINTNL(gui_input_spell),0,0,0},
+#endif
 
+	{ST_HEADER, N_("Nick Completion"),0,0,0},
+	{ST_TOGGLE, N_("Automatic nick completion (without TAB key)"), P_OFFINTNL(nickcompletion),
+					0,0,0},
+	{ST_ENTRY,	N_("Nick completion suffix:"), P_OFFSETNL(nick_suffix),0,0,sizeof prefs.nick_suffix},
+	{ST_MENU,	N_("Nick completion sorted:"), P_OFFINTNL(completion_sort), 0, tabcompmenu, 0},
+
+#if 0	/* obsolete */
 	{ST_HEADER, N_("Input Box Codes"),0,0,0},
 	{ST_TOGGLE, N_("Interpret %nnn as an ASCII value"), P_OFFINTNL(perc_ascii),0,0,0},
 	{ST_TOGGLE, N_("Interpret %C, %B as Color, Bold etc"), P_OFFINTNL(perc_color),0,0,0},
-
-	{ST_HEADER, N_("Input Box Appearance"),0,0,0},
-	{ST_TOGGLE, N_("Use the Text box font and colors"), P_OFFINTNL(style_inputbox),0,0,0},
+#endif
 
 	{ST_END, 0, 0, 0, 0, 0}
 };
@@ -157,13 +173,13 @@ static const char *const ulmenutext[] =
 static const setting userlist_settings[] =
 {
 	{ST_HEADER,	N_("User List"),0,0,0},
-	{ST_MENU,	N_("User list sorted by:"), P_OFFINTNL(userlist_sort), 0, ulmenutext, 0},
 	{ST_TOGGLE, N_("Show hostnames in user list"), P_OFFINTNL(showhostname_in_userlist), 0, 0, 0},
 	{ST_TOGGLE, N_("Use the Text box font and colors"), P_OFFINTNL(style_namelistgad),0,0,0},
 	{ST_TOGGLE, N_("Resizable user list"), P_OFFINTNL(paned_userlist),0,0,0},
+	{ST_MENU,	N_("User list sorted by:"), P_OFFINTNL(userlist_sort), 0, ulmenutext, 0},
 
 	{ST_HEADER,	N_("Away tracking"),0,0,0},
-	{ST_TOGGLE,	N_("Track the Away status of users and mark them in a different color"), P_OFFINTNL(away_track),0,0,0},
+	{ST_TOGGLE,	N_("Track the Away status of users and mark them in a different color"), P_OFFINTNL(away_track),0,0,2},
 	{ST_NUMBER, N_("On channels smaller than:"), P_OFFINTNL(away_size_max),0,0,10000},
 
 	{ST_HEADER,	N_("Action Upon Double Click"),0,0,0},
@@ -206,13 +222,12 @@ static const setting tabs_settings[] =
 	{ST_HEADER,	N_("Tabs"),0,0,0},
 	{ST_TOGGLE, N_("Open an extra tab for server messages"), P_OFFINTNL(use_server_tab), 0, 0, 0},
 	{ST_TOGGLE, N_("Open an extra tab for server notices"), P_OFFINTNL(notices_tabs), 0, 0, 0},
-	{ST_TOGGLE, N_("Small tabs"), P_OFFINTNL(tab_small), 0, 0, 0},
+	{ST_TOGGLE, N_("Open a new tab when you receive a private message"), P_OFFINTNL(autodialog), 0, 0, 0},
 	{ST_TOGGLE, N_("Sort tabs in alphabetical order"), P_OFFINTNL(tab_sort), 0, 0, 0},
+	{ST_TOGGLE, N_("Small tabs"), P_OFFINTNL(tab_small), 0, 0, 0},
 	{ST_MENU,	N_("Focus new tabs:"), P_OFFINTNL(newtabstofront), 0, focusnewtabsmenu, 0},
-	{ST_NUMBER,	N_("Shorten tab labels to:"), P_OFFINTNL(truncchans), 0, (const char **)N_("letters."), 99},
-
-	{ST_HEADER,	N_("Tabs Location"),0,0,0},
 	{ST_MENU,	N_("Show tabs at:"), P_OFFINTNL(tabs_position), 0, tabpos, 0},
+	{ST_NUMBER,	N_("Shorten tab labels to:"), P_OFFINTNL(truncchans), 0, (const char **)N_("letters."), 99},
 
 	{ST_HEADER,	N_("Tabs or Windows"),0,0,0},
 	{ST_MENU,	N_("Open channels in:"), P_OFFINTNL(tabchannels), 0, tabwin, 0},
@@ -222,23 +237,30 @@ static const setting tabs_settings[] =
 	{ST_END, 0, 0, 0, 0, 0}
 };
 
+static const char *const dccaccept[] =
+{
+	N_("No"),
+	N_("Yes"),
+	N_("Browse for save folder every time"),
+	NULL
+};
+
 static const setting filexfer_settings[] =
 {
 	{ST_HEADER, N_("Files and Directories"), 0, 0, 0},
-	{ST_EOPEN,	N_("Download files to:"), P_OFFSETNL(dccdir), 0, 0, sizeof prefs.dccdir},
-	{ST_EOPEN,	N_("Move completed files to:"), P_OFFSETNL(dcc_completed_dir), 0, 0, sizeof prefs.dcc_completed_dir},
-	{ST_TOGGLE, N_("Auto accept file offers"), P_OFFINTNL(autodccsend), 0, 0, 0},
+	{ST_MENU,	N_("Auto accept file offers:"), P_OFFINTNL(autodccsend), 0, dccaccept, 0},
+	{ST_EFOLDER,N_("Download files to:"), P_OFFSETNL(dccdir), 0, 0, sizeof prefs.dccdir},
+	{ST_EFOLDER,N_("Move completed files to:"), P_OFFSETNL(dcc_completed_dir), 0, 0, sizeof prefs.dcc_completed_dir},
 	{ST_TOGGLE, N_("Save nick name in filenames"), P_OFFINTNL(dccwithnick), 0, 0, 0},
-	{ST_TOGGLE, N_("Convert spaces to underscore before sending"), P_OFFINTNL(dcc_send_fillspaces),0,0,0},
 
 	{ST_HEADER, N_("Network Settings"), 0, 0, 0},
+	{ST_TOGGLE, N_("Get my address from the IRC server"), P_OFFINTNL(ip_from_server),
+					N_("Asks the IRC server for your real address. Use this if you have a 192.168.*.* address!"), 0, 0},
 	{ST_ENTRY,	N_("DCC IP address:"), P_OFFSETNL(dcc_ip_str),
 					N_("Claim you are at this address when offering files."), 0, sizeof prefs.dcc_ip_str},
 	{ST_NUMBER,	N_("First DCC send port:"), P_OFFINTNL(first_dcc_send_port), 0, 0, 65535},
 	{ST_NUMBER,	N_("Last DCC send port:"), P_OFFINTNL(last_dcc_send_port), 0, 
-		(const char **)N_("(Leave ports at zero for full range)."), 65535},
-	{ST_TOGGLE, N_("Get my address from the IRC server"), P_OFFINTNL(ip_from_server),
-					N_("Asks the IRC server for your real address. Use this if you have a 192.168.*.* address!"), 0, 0},
+		(const char **)N_("!Leave ports at zero for full range."), 65535},
 
 	{ST_HEADER, N_("Maximum File Transfer Speeds (bytes per second)"), 0, 0, 0},
 	{ST_NUMBER,	N_("One upload:"), P_OFFINTNL(dcc_max_send_cps), 
@@ -275,14 +297,15 @@ static const setting general_settings[] =
 
 	{ST_HEADER,	N_("Alerts"),0,0,0},
 #if defined(WIN32) || defined(USE_XLIB)
-	{ST_TOGGLE,	N_("Flash taskbar on highlighted messages"), P_OFFINTNL(flash_hilight), 0, 0, 0},
+	{ST_TOGGLE,	N_("Flash taskbar on highlighted messages"), P_OFFINTNL(input_flash_hilight), 0, 0, 0},
+	{ST_TOGGLE,	N_("Flash taskbar on any private messages"), P_OFFINTNL(input_flash_priv), 0, 0, 0},
 #endif
 	{ST_TOGGLE,	N_("Beep on highlighted messages"), P_OFFINTNL(beephilight), 0, 0, 0},
-	{ST_ENTRY,	N_("Extra words to highlight on:"), P_OFFSETNL(irc_extra_hilight), 0, 0, sizeof prefs.irc_extra_hilight},
-	{ST_ENTRY,	N_("Nicks not to highlight on:"), P_OFFSETNL(irc_no_hilight), 0, 0, sizeof prefs.irc_no_hilight},
-	{ST_LABEL,	N_("(Separate multiple words with commas).")},
 	{ST_TOGGLE,	N_("Beep on private messages"), P_OFFINTNL(beepmsg), 0, 0, 0},
 	{ST_TOGGLE,	N_("Beep on channel messages"), P_OFFINTNL(beepchans), 0, 0, 0},
+	{ST_ENTRY,	N_("Extra words to highlight on:"), P_OFFSETNL(irc_extra_hilight), 0, 0, sizeof prefs.irc_extra_hilight},
+	{ST_ENTRY,	N_("Nicks not to highlight on:"), P_OFFSETNL(irc_no_hilight), 0, 0, sizeof prefs.irc_no_hilight},
+	{ST_LABEL,	N_("Separate multiple words with commas.")},
 	{ST_END, 0, 0, 0, 0, 0}
 };
 
@@ -306,14 +329,14 @@ static const setting advanced_settings[] =
 static const setting logging_settings[] =
 {
 	{ST_HEADER,	N_("Logging"),0,0,0},
-	{ST_TOGGLE,	N_("Enable logging of conversations"), P_OFFINTNL(logging), 0, 0, 0},
+	{ST_TOGGLE,	N_("Enable logging of conversations"), P_OFFINTNL(logging), 0, 0, 2},
 	{ST_ENTRY,	N_("Log filename:"), P_OFFSETNL(logmask), 0, 0, sizeof prefs.logmask},
-	{ST_LABEL,	N_("(%s=Server %c=Channel %n=Network).")},
+	{ST_LABEL,	N_("%s=Server %c=Channel %n=Network.")},
 
 	{ST_HEADER,	N_("Time Stamps"),0,0,0},
-	{ST_TOGGLE,	N_("Insert timestamps in logs"), P_OFFINTNL(timestamp_logs), 0, 0, 0},
+	{ST_TOGGLE,	N_("Insert timestamps in logs"), P_OFFINTNL(timestamp_logs), 0, 0, 2},
 	{ST_ENTRY,	N_("Log timestamp format:"), P_OFFSETNL(timestamp_log_format), 0, 0, sizeof prefs.timestamp_log_format},
-	{ST_LABEL,	N_("(See strftime manpage for details).")},
+	{ST_LABEL,	N_("See strftime manpage for details.")},
 
 	{ST_END, 0, 0, 0, 0, 0}
 };
@@ -325,6 +348,17 @@ static const char *const proxytypes[] =
 	N_("Socks4"),
 	N_("Socks5"),
 	N_("HTTP"),
+#ifdef USE_MSPROXY
+	N_("MS Proxy (ISA)"),
+#endif
+	NULL
+};
+
+static const char *const proxyuse[] =
+{
+	N_("All Connections"),
+	N_("IRC Server Only"),
+	N_("DCC Get Only"),
 	NULL
 };
 
@@ -332,16 +366,22 @@ static const setting network_settings[] =
 {
 	{ST_HEADER,	N_("Your Address"), 0, 0, 0, 0},
 	{ST_ENTRY,	N_("Bind to:"), P_OFFSETNL(hostname), 0, 0, sizeof prefs.hostname},
-	{ST_LABEL,	N_("(Only useful for computers with multiple addresses).")},
+	{ST_LABEL,	N_("Only useful for computers with multiple addresses.")},
 
 	{ST_HEADER,	N_("Proxy Server"), 0, 0, 0, 0},
 	{ST_ENTRY,	N_("Hostname:"), P_OFFSETNL(proxy_host), 0, 0, sizeof prefs.proxy_host},
-	{ST_ENTRY,	N_("Username:"), P_OFFSETNL(proxy_user), 0, 0, sizeof prefs.proxy_user},
-	{ST_ENTRY,	N_("Password:"), P_OFFSETNL(proxy_pass), 0, GINT_TO_POINTER(1), sizeof prefs.proxy_pass},
 	{ST_NUMBER,	N_("Port:"), P_OFFINTNL(proxy_port), 0, 0, 65535},
 	{ST_MENU,	N_("Type:"), P_OFFINTNL(proxy_type), 0, proxytypes, 0},
+	{ST_MENU,	N_("Use proxy for:"), P_OFFINTNL(proxy_use), 0, proxyuse, 0},
 
-	{ST_TOGGLE,	N_("Authenticate to the proxy server (HTTP and Socks5)"), P_OFFINTNL(proxy_auth), 0, 0, 0},
+	{ST_HEADER,	N_("Proxy Authentication"), 0, 0, 0, 0},
+#ifdef USE_MSPROXY
+	{ST_TOGGLE,	N_("Use Authentication (MS Proxy, HTTP or Socks5 only)"), P_OFFINTNL(proxy_auth), 0, 0, 0},
+#else
+	{ST_TOGGLE,	N_("Use Authentication (HTTP or Socks5 only)"), P_OFFINTNL(proxy_auth), 0, 0, 0},
+#endif
+	{ST_ENTRY,	N_("Username:"), P_OFFSETNL(proxy_user), 0, 0, sizeof prefs.proxy_user},
+	{ST_ENTRY,	N_("Password:"), P_OFFSETNL(proxy_pass), 0, GINT_TO_POINTER(1), sizeof prefs.proxy_pass},
 
 	{ST_END, 0, 0, 0, 0, 0}
 };
@@ -356,7 +396,18 @@ static const setting network_settings[] =
 static void
 setup_toggle_cb (GtkToggleButton *but, const setting *set)
 {
+	GtkWidget *label, *disable_wid;
+
 	setup_set_int (&setup_prefs, set, but->active ? 1 : 0);
+
+	/* does this toggle also enable/disable another widget? */
+	disable_wid = g_object_get_data (G_OBJECT (but), "nxt");
+	if (disable_wid)
+	{
+		gtk_widget_set_sensitive (disable_wid, but->active);
+		label = g_object_get_data (G_OBJECT (disable_wid), "lbl");
+		gtk_widget_set_sensitive (label, but->active);
+	}
 }
 
 static void
@@ -372,10 +423,10 @@ setup_create_toggleR (GtkWidget *tab, int row, const setting *set)
 	if (set->tooltip)
 		add_tip (wid, _(set->tooltip));
 	gtk_table_attach (GTK_TABLE (tab), wid, 4, 5, row, row + 1,
-							GTK_SHRINK | GTK_FILL, GTK_SHRINK | GTK_FILL, 0, 0);
+							GTK_EXPAND | GTK_SHRINK | GTK_FILL, GTK_SHRINK | GTK_FILL, 0, 0);
 }
 
-static void
+static GtkWidget *
 setup_create_toggleL (GtkWidget *tab, int row, const setting *set)
 {
 	GtkWidget *wid;
@@ -387,8 +438,10 @@ setup_create_toggleL (GtkWidget *tab, int row, const setting *set)
 							G_CALLBACK (setup_toggle_cb), (gpointer)set);
 	if (set->tooltip)
 		add_tip (wid, _(set->tooltip));
-	gtk_table_attach (GTK_TABLE (tab), wid, 2, 4, row, row + 1,
+	gtk_table_attach (GTK_TABLE (tab), wid, 2, row==6 ? 6 : 4, row, row + 1,
 							GTK_SHRINK | GTK_FILL, GTK_SHRINK | GTK_FILL, LABEL_INDENT, 0);
+
+	return wid;
 }
 
 #if 0
@@ -408,16 +461,30 @@ setup_create_toggle (GtkWidget *box, int row, const setting *set)
 }
 #endif
 
+static GtkWidget *
+setup_create_italic_label (char *text)
+{
+	GtkWidget *label;
+	char buf[256];
+
+	label = gtk_label_new (NULL);
+	snprintf (buf, sizeof (buf), "<i><span size=\"smaller\">%s</span></i>", text);
+	gtk_label_set_markup (GTK_LABEL (label), buf);
+
+	return label;
+}
+
 static void
 setup_spin_cb (GtkSpinButton *spin, const setting *set)
 {
 	setup_set_int (&setup_prefs, set, gtk_spin_button_get_value_as_int (spin));
 }
 
-static void
+static GtkWidget *
 setup_create_spin (GtkWidget *table, int row, const setting *set)
 {
 	GtkWidget *label, *wid, *rbox, *align;
+	char *text;
 
 	label = gtk_label_new (_(set->label));
 	gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
@@ -426,12 +493,13 @@ setup_create_spin (GtkWidget *table, int row, const setting *set)
 
 	align = gtk_alignment_new (0.0, 0.5, 0.0, 0.0);
 	gtk_table_attach (GTK_TABLE (table), align, 3, 4, row, row + 1,
-							GTK_SHRINK | GTK_FILL, GTK_SHRINK | GTK_FILL, 0, 0);
+							GTK_EXPAND | GTK_FILL, GTK_SHRINK | GTK_FILL, 0, 0);
 
 	rbox = gtk_hbox_new (0, 0);
 	gtk_container_add (GTK_CONTAINER (align), rbox);
 
 	wid = gtk_spin_button_new_with_range (0, set->extra, 1);
+	g_object_set_data (G_OBJECT (wid), "lbl", label);
 	if (set->tooltip)
 		add_tip (wid, _(set->tooltip));
 	gtk_spin_button_set_value (GTK_SPIN_BUTTON (wid),
@@ -442,9 +510,15 @@ setup_create_spin (GtkWidget *table, int row, const setting *set)
 
 	if (set->list)
 	{
-		label = gtk_label_new (_((char *)set->list));
-		gtk_box_pack_start (GTK_BOX (rbox), label, 0, 0, 5);
+		text = _((char *)set->list);
+		if (text[0] == '!')
+			label = setup_create_italic_label (text + 1);
+		else
+			label = gtk_label_new (text);
+		gtk_box_pack_start (GTK_BOX (rbox), label, 0, 0, 6);
 	}
+
+	return wid;
 }
 
 static gint
@@ -489,17 +563,17 @@ setup_create_hscale (GtkWidget *table, int row, const setting *set)
 
 #if 0
 static int
-setup_create_radio (GtkWidget *table, int row, setting *set)
+setup_create_radio (GtkWidget *table, int row, const setting *set)
 {
 	GtkWidget *wid;
 	int i;
-	char **text = set->list;
+	const char **text = (const char **)set->list;
 	GSList *group;
 
-	wid = gtk_label_new (set->label);
+	wid = gtk_label_new (_(set->label));
 	gtk_misc_set_alignment (GTK_MISC (wid), 0.0, 0.5);
 	gtk_table_attach (GTK_TABLE (table), wid, 2, 3, row, row + 1,
-							GTK_SHRINK | GTK_FILL, GTK_SHRINK | GTK_FILL, 0, 0);
+							GTK_SHRINK | GTK_FILL, GTK_SHRINK | GTK_FILL, LABEL_INDENT, 0);
 
 	i = 0;
 	group = NULL;
@@ -509,7 +583,7 @@ setup_create_radio (GtkWidget *table, int row, setting *set)
 		if (set->tooltip)
 			add_tip (wid, _(set->tooltip));
 		group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (wid));
-		gtk_table_attach_defaults (GTK_TABLE (table), wid, 3, 5, row, row + 1);
+		gtk_table_attach_defaults (GTK_TABLE (table), wid, 3, 4, row, row + 1);
 		if (i == setup_get_int (&setup_prefs, set))
 			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (wid), TRUE);
 		i++;
@@ -525,18 +599,18 @@ static GtkWidget *proxy_pass; 	/* password GtkEntry */
 
 
 static void
-setup_menu_cb (GtkWidget *item, const setting *set)
+setup_menu_cb (GtkWidget *cbox, const setting *set)
 {
-	int n = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (item), "n"));
+	int n = gtk_combo_box_get_active (GTK_COMBO_BOX (cbox));
 
 	/* set the prefs.<field> */
 	setup_set_int (&setup_prefs, set, n);
 
-	if (g_object_get_data (G_OBJECT (item), "p"))
+	if (set->list == proxytypes)
 	{
 		/* only HTTP and Socks5 can use a username/pass */
-		gtk_widget_set_sensitive (proxy_user, (n == 3 || n == 4));
-		gtk_widget_set_sensitive (proxy_pass, (n == 3 || n == 4));
+		gtk_widget_set_sensitive (proxy_user, (n == 3 || n == 4 || n == 5));
+		gtk_widget_set_sensitive (proxy_pass, (n == 3 || n == 4 || n == 5));
 	}
 }
 
@@ -564,10 +638,10 @@ setup_create_id_menu (GtkWidget *table, char *label, int row, char *dest)
 	int i, def = 0;
 	static const char *text[] =
 	{
-		N_("(disabled)"),
-		N_("A star (*)"),
-		N_("A red star (*)"),
-		N_("Underlined")
+		("(disabled)"),
+		("A star (*)"),
+		("A red star (*)"),
+		("Underlined")
 	};
 
 	wid = gtk_label_new (label);
@@ -612,43 +686,29 @@ setup_create_id_menu (GtkWidget *table, char *label, int row, char *dest)
 static void
 setup_create_menu (GtkWidget *table, int row, const setting *set)
 {
-	GtkWidget *wid, *menu, *item;
-	int i;
+	GtkWidget *wid, *cbox, *box;
 	const char **text = (const char **)set->list;
+	int i;
 
 	wid = gtk_label_new (_(set->label));
 	gtk_misc_set_alignment (GTK_MISC (wid), 0.0, 0.5);
 	gtk_table_attach (GTK_TABLE (table), wid, 2, 3, row, row + 1,
 							GTK_SHRINK | GTK_FILL, GTK_SHRINK | GTK_FILL, LABEL_INDENT, 0);
 
-	wid = gtk_option_menu_new ();
-	if (set->tooltip)
-		add_tip (wid, _(set->tooltip));
-	menu = gtk_menu_new ();
+	cbox = gtk_combo_box_new_text ();
 
-	i = 0;
-	while (text[i])
-	{
-		item = gtk_menu_item_new_with_label (_(text[i]));
-		g_object_set_data (G_OBJECT (item), "n", GINT_TO_POINTER (i));
+	for (i = 0; text[i]; i++)
+		gtk_combo_box_append_text (GTK_COMBO_BOX (cbox), _(text[i]));
 
-		/* set a flag for the callback to use */
-		if (text == proxytypes)
-			g_object_set_data (G_OBJECT (item), "p", GINT_TO_POINTER (1));	
+	gtk_combo_box_set_active (GTK_COMBO_BOX (cbox),
+									  setup_get_int (&setup_prefs, set));
+	g_signal_connect (G_OBJECT (cbox), "changed",
+							G_CALLBACK (setup_menu_cb), (gpointer)set);
 
-		gtk_widget_show (item);
-		gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
-		g_signal_connect (G_OBJECT (item), "activate",
-								G_CALLBACK (setup_menu_cb), (gpointer)set);
-		i++;
-	}
-
-	gtk_option_menu_set_menu (GTK_OPTION_MENU (wid), menu);
-	gtk_option_menu_set_history (GTK_OPTION_MENU (wid),
-										  setup_get_int (&setup_prefs, set));
-
-	gtk_table_attach (GTK_TABLE (table), wid, 3, 4, row, row + 1,
-							GTK_SHRINK | GTK_FILL, GTK_SHRINK | GTK_FILL, 0, 0);
+	box = gtk_hbox_new (0, 0);
+	gtk_box_pack_start (GTK_BOX (box), cbox, 0, 0, 0);
+	gtk_table_attach (GTK_TABLE (table), box, 3, 4, row, row + 1,
+							GTK_EXPAND | GTK_FILL, GTK_SHRINK | GTK_FILL, 0, 0);
 }
 
 static void
@@ -685,14 +745,11 @@ setup_fontsel_cancel (GtkWidget *button, GtkFontSelectionDialog *dialog)
 	gtk_widget_destroy (GTK_WIDGET (dialog));
 }
 
-#ifdef WIN32
 static void
-setup_browsefolder_cb (GtkWidget *button, GtkWidget *entry)
+setup_browsefolder_cb (GtkWidget *button, GtkEntry *entry)
 {
-	if (GTK_ENTRY (entry)->text[0])
-		fe_open_url (GTK_ENTRY (entry)->text);
+	gtkutil_file_req (_("Select Download Folder"), setup_filereq_cb, entry, entry->text, FRF_CHOOSEFOLDER);
 }
-#endif
 
 static void
 setup_browsefont_cb (GtkWidget *button, GtkWidget *entry)
@@ -726,12 +783,12 @@ setup_entry_cb (GtkEntry *entry, setting *set)
 static void
 setup_create_label (GtkWidget *table, int row, const setting *set)
 {
-	gtk_table_attach (GTK_TABLE (table), gtk_label_new (_(set->label)),
-							3, 4, row, row + 1, GTK_FILL,
+	gtk_table_attach (GTK_TABLE (table), setup_create_italic_label (_(set->label)),
+							3, 5, row, row + 1, GTK_FILL,
 							GTK_SHRINK | GTK_FILL, 0, 0);
 }
 
-static void
+static GtkWidget *
 setup_create_entry (GtkWidget *table, int row, const setting *set)
 {
 	GtkWidget *label;
@@ -743,6 +800,7 @@ setup_create_entry (GtkWidget *table, int row, const setting *set)
 							GTK_SHRINK | GTK_FILL, GTK_SHRINK | GTK_FILL, LABEL_INDENT, 0);
 
 	wid = gtk_entry_new ();
+	g_object_set_data (G_OBJECT (wid), "lbl", label);
 	if (set->list)
 		gtk_entry_set_visibility (GTK_ENTRY (wid), FALSE);
 	if (set->tooltip)
@@ -760,35 +818,31 @@ setup_create_entry (GtkWidget *table, int row, const setting *set)
 	/* only http and Socks5 can auth */
 	if ( (set->offset == P_OFFSETNL(proxy_pass) ||
 			set->offset == P_OFFSETNL(proxy_user)) &&
-	     (setup_prefs.proxy_type != 4 && setup_prefs.proxy_type != 3) )
+	     (setup_prefs.proxy_type != 4 && setup_prefs.proxy_type != 3 && setup_prefs.proxy_type != 5) )
 		gtk_widget_set_sensitive (wid, FALSE);
 
-#ifdef WIN32
 	if (set->type == ST_ENTRY)
-#else
-	if (set->type == ST_ENTRY || set->type == ST_EOPEN)
-#endif
 		gtk_table_attach (GTK_TABLE (table), wid, 3, 6, row, row + 1,
 								GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 0);
 	else
 	{
 		gtk_table_attach (GTK_TABLE (table), wid, 3, 5, row, row + 1,
-								GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 0);
+								GTK_EXPAND | GTK_FILL, GTK_SHRINK, 0, 0);
 		bwid = gtk_button_new_with_label (_("Browse..."));
 		gtk_table_attach (GTK_TABLE (table), bwid, 5, 6, row, row + 1,
-								GTK_SHRINK | GTK_FILL, GTK_SHRINK, 0, 0);
+								GTK_SHRINK | GTK_FILL, GTK_FILL, 0, 0);
 		if (set->type == ST_EFILE)
 			g_signal_connect (G_OBJECT (bwid), "clicked",
 									G_CALLBACK (setup_browsefile_cb), wid);
 		if (set->type == ST_EFONT)
 			g_signal_connect (G_OBJECT (bwid), "clicked",
 									G_CALLBACK (setup_browsefont_cb), wid);
-#ifdef WIN32
-		if (set->type == ST_EOPEN)
+		if (set->type == ST_EFOLDER)
 			g_signal_connect (G_OBJECT (bwid), "clicked",
 									G_CALLBACK (setup_browsefolder_cb), wid);
-#endif
 	}
+
+	return wid;
 }
 
 static void
@@ -797,7 +851,10 @@ setup_create_header (GtkWidget *table, int row, char *labeltext)
 	GtkWidget *label;
 	char buf[128];
 
-	snprintf (buf, sizeof (buf), "<b>%s</b>", _(labeltext));
+	if (row == 0)
+		snprintf (buf, sizeof (buf), "<b>%s</b>", _(labeltext));
+	else
+		snprintf (buf, sizeof (buf), "\n<b>%s</b>", _(labeltext));
 
 	label = gtk_label_new (NULL);
 	gtk_label_set_markup (GTK_LABEL (label), buf);
@@ -812,7 +869,7 @@ setup_create_frame (GtkWidget **left, GtkWidget *box)
 	GtkWidget *tab, *hbox, *inbox = box;
 
 	tab = gtk_table_new (3, 2, FALSE);
-	gtk_container_set_border_width (GTK_CONTAINER (tab), 2);
+	gtk_container_set_border_width (GTK_CONTAINER (tab), 6);
 	gtk_table_set_row_spacings (GTK_TABLE (tab), 2);
 	gtk_table_set_col_spacings (GTK_TABLE (tab), 3);
 	gtk_container_add (GTK_CONTAINER (inbox), tab);
@@ -826,28 +883,29 @@ setup_create_frame (GtkWidget **left, GtkWidget *box)
 	return tab;
 }
 
-#ifdef WIN32
 static void
 open_data_cb (GtkWidget *button, gpointer data)
 {
-	ShellExecute (0, "open", get_xdir_fs (), NULL, NULL, SW_SHOWNORMAL);
+	fe_open_url (get_xdir_utf8 ());
 }
-#endif
 
 static GtkWidget *
 setup_create_page (const setting *set)
 {
-	int i, j, row;
+	int i, row, do_disable;
 	GtkWidget *tab, *box, *left;
+	GtkWidget *wid = NULL, *prev;
 
-	box = gtk_vbox_new (FALSE, 20);
-	gtk_container_set_border_width (GTK_CONTAINER (box), 4);
+	box = gtk_vbox_new (FALSE, 1);
+	gtk_container_set_border_width (GTK_CONTAINER (box), 6);
 
 	tab = setup_create_frame (&left, box);
 
-	i = j = row = 0;
+	i = row = do_disable = 0;
 	while (set[i].type != ST_END)
 	{
+		prev = wid;
+
 		switch (set[i].type)
 		{
 		case ST_HEADER:
@@ -856,15 +914,16 @@ setup_create_page (const setting *set)
 		case ST_EFONT:
 		case ST_ENTRY:
 		case ST_EFILE:
-		case ST_EOPEN:
-			setup_create_entry (tab, row, &set[i]);
+		case ST_EFOLDER:
+			wid = setup_create_entry (tab, row, &set[i]);
 			break;
 		case ST_TOGGLR:
 			row--;
 			setup_create_toggleR (tab, row, &set[i]);
 			break;
 		case ST_TOGGLE:
-			setup_create_toggleL (tab, row, &set[i]);
+			wid = setup_create_toggleL (tab, row, &set[i]);
+			do_disable = set[i].extra;
 			break;
 		case ST_MENU:
 			setup_create_menu (tab, row, &set[i]);
@@ -875,7 +934,7 @@ setup_create_page (const setting *set)
 			break;
 #endif
 		case ST_NUMBER:
-			setup_create_spin (tab, row, &set[i]);
+			wid = setup_create_spin (tab, row, &set[i]);
 			break;
 		case ST_HSCALE:
 			setup_create_hscale (tab, row, &set[i]);
@@ -884,6 +943,19 @@ setup_create_page (const setting *set)
 			setup_create_label (tab, row, &set[i]);
 			break;
 		}
+
+		/* will this toggle disable the "next" widget? */
+		do_disable--;
+		if (do_disable == 0)
+		{
+			/* setup_toggle_cb uses this data */
+			g_object_set_data (G_OBJECT (prev), "nxt", wid);
+			/* force initial sensitive state */
+			gtk_widget_set_sensitive (wid, GTK_TOGGLE_BUTTON (prev)->active);
+			gtk_widget_set_sensitive (g_object_get_data (G_OBJECT (wid), "lbl"),
+											  GTK_TOGGLE_BUTTON (prev)->active);
+		}
+
 		i++;
 		row++;
 	}
@@ -898,15 +970,13 @@ setup_create_page (const setting *set)
 	}
 #endif
 
-#ifdef WIN32
 	if (set == logging_settings)
 	{
-		GtkWidget *but = gtk_button_new_with_label ("Open Data Folder");
+		GtkWidget *but = gtk_button_new_with_label (_("Open Data Folder"));
 		gtk_box_pack_start (GTK_BOX (left), but, 0, 0, 0);
 		g_signal_connect (G_OBJECT (but), "clicked",
 								G_CALLBACK (open_data_cb), 0);
 	}
-#endif
 
 	return box;
 }
@@ -1028,10 +1098,10 @@ setup_create_color_page (void)
 	int i;
 
 	box = gtk_vbox_new (FALSE, 0);
-	gtk_container_set_border_width (GTK_CONTAINER (box), 4);
+	gtk_container_set_border_width (GTK_CONTAINER (box), 6);
 
 	tab = gtk_table_new (9, 2, FALSE);
-	gtk_container_set_border_width (GTK_CONTAINER (tab), 2);
+	gtk_container_set_border_width (GTK_CONTAINER (tab), 6);
 	gtk_table_set_row_spacings (GTK_TABLE (tab), 2);
 	gtk_table_set_col_spacings (GTK_TABLE (tab), 3);
 	gtk_container_add (GTK_CONTAINER (box), tab);
@@ -1046,7 +1116,7 @@ setup_create_color_page (void)
 	for (i = 0; i < 16; i++)
 		setup_create_color_button (tab, i, 1, i+3);
 
-	label = gtk_label_new (_("Extra colors:"));
+	label = gtk_label_new (_("Local colors:"));
 	gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
 	gtk_table_attach (GTK_TABLE (tab), label, 2, 3, 2, 3,
 							GTK_SHRINK | GTK_FILL, GTK_SHRINK | GTK_FILL, LABEL_INDENT, 0);
@@ -1211,7 +1281,7 @@ setup_snd_browse_cb (GtkWidget *button, GtkEntry *entry)
 static void
 setup_snd_play_cb (GtkWidget *button, GtkEntry *entry)
 {
-	sound_play (entry->text);
+	sound_play (entry->text, FALSE);
 }
 
 static void
@@ -1589,6 +1659,10 @@ setup_apply_entry_style (GtkWidget *entry)
 static void
 setup_apply_to_sess (session_gui *gui)
 {
+#ifdef USE_GTKSPELL
+	GtkSpell *spell;
+#endif
+
 	mg_update_xtext (gui->xtext);
 
 	if (prefs.style_namelistgad)
@@ -1614,6 +1688,39 @@ setup_apply_to_sess (session_gui *gui)
 		gtk_widget_show (gui->button_box);
 	else
 		gtk_widget_hide (gui->button_box);
+
+#ifdef USE_GTKSPELL
+	spell = gtkspell_get_from_text_view (GTK_TEXT_VIEW (gui->input_box));
+	if (prefs.gui_input_spell)
+	{
+		if (!spell)
+			gtkspell_new_attach (GTK_TEXT_VIEW (gui->input_box), NULL, NULL);
+	}
+	else
+	{
+		if (spell)
+			gtkspell_detach (spell);
+	}
+#endif
+
+#ifdef USE_LIBSEXY
+	sexy_spell_entry_set_checked ((SexySpellEntry *)gui->input_box, prefs.gui_input_spell);
+#endif
+}
+
+static void
+unslash (char *dir)
+{
+	if (dir[0])
+	{
+		int len = strlen (dir) - 1;
+#ifdef WIN32
+		if (dir[len] == '/' || dir[len] == '\\')
+#else
+		if (dir[len] == '/')
+#endif
+			dir[len] = 0;
+	}
 }
 
 static void
@@ -1647,12 +1754,18 @@ setup_apply (struct xchatprefs *pr)
 		noapply = TRUE;
 	if (DIFF (style_namelistgad))
 		noapply = TRUE;
+	if (DIFF (truncchans))
+		noapply = TRUE;
 
 	if (color_change || (DIFF (away_size_max)) || (DIFF (away_track)))
 		do_ulist = TRUE;
 
 	memcpy (&prefs, pr, sizeof (prefs));
 
+	/* remove trailing slashes */
+	unslash (prefs.dccdir);
+	unslash (prefs.dcc_completed_dir);
+
 	mkdir_utf8 (prefs.dccdir);
 	mkdir_utf8 (prefs.dcc_completed_dir);
 
@@ -1698,6 +1811,19 @@ setup_apply (struct xchatprefs *pr)
 	if (noapply)
 		fe_message (_("Some settings were changed that require a"
 						" restart to take full effect."), FE_MSG_WARN);
+
+#ifndef WIN32
+	if (prefs.autodccsend == 1)
+	{
+		if (!strcmp ((char *)g_get_home_dir (), prefs.dccdir))
+		{
+			fe_message (_("*WARNING*\n"
+							 "Auto accepting DCC to your home directory\n"
+							 "can be dangerous and is exploitable. Eg:\n"
+							 "Someone could send you a .bash_profile"), FE_MSG_WARN);
+		}
+	}
+#endif
 }
 
 #if 0
