@@ -99,43 +99,25 @@ fe_new_window (struct session *sess, int focus)
 		return;
 	done_intro = 1;
 
-	snprintf (buf, sizeof (buf),	
+	snprintf (buf, sizeof (buf),
 				"\n"
-				" \017xchat \00310"VERSION"\n"
+				" \017xchat \00310"PACKAGE_VERSION"\n"
 				" \017Running on \00310%s \017glib \00310%d.%d.%d\n"
 				" \017This binary compiled \00310"__DATE__"\017\n",
-				get_cpu_str(), 
+				get_cpu_str(),
 				glib_major_version, glib_minor_version, glib_micro_version);
 	fe_print_text (sess, buf);
 
 	fe_print_text (sess, "\n\nCompiled in Features\0032:\017 "
-#ifdef USE_PERL
-	"Perl "
-#endif
-#ifdef USE_PYTHON
-	"Python "
-#endif
 #ifdef USE_PLUGIN
 	"Plugin "
 #endif
 #ifdef ENABLE_NLS
 	"NLS "
 #endif
-#ifdef USE_TRANS
-	"Trans "
-#endif
-#ifdef USE_HEBREW
-	"Hebrew "
-#endif
 #ifdef USE_OPENSSL
 	"OpenSSL "
 #endif
-#ifdef SOCKS
-	"Socks5 "
-#endif
-#ifdef USE_JCODE
-	"JCode "
-#endif
 #ifdef USE_IPV6
 	"IPv6"
 #endif
@@ -411,11 +393,11 @@ fe_args (int argc, char *argv[])
 	{
 		if (!strcasecmp (argv[1], "--version") || !strcasecmp (argv[1], "-v"))
 		{
-			puts (VERSION);
+			puts (PACKAGE_VERSION);
 			return 0;
 		}
 	}
-	return 1;
+	return -1;
 }
 
 void
@@ -446,8 +428,9 @@ fe_main (void)
 		new_ircwindow (NULL, NULL, SESS_SERVER, 0);
 
 #ifdef ENABLE_NLS
-	bindtextdomain (PACKAGE, PREFIX"/share/locale");
-	textdomain (PACKAGE);
+	bindtextdomain (GETTEXT_PACKAGE, PREFIX"/share/locale");
+	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
+	textdomain (GETTEXT_PACKAGE);
 #endif
 
 	while (!done)
@@ -566,7 +549,7 @@ fe_new_server (struct server *serv)
 }
 
 void
-fe_message (char *msg, int wait)
+fe_message (char *msg, int flags)
 {
 	puts (msg);
 }
@@ -574,7 +557,7 @@ fe_message (char *msg, int wait)
 void
 fe_close_window (struct session *sess)
 {
-	kill_session_callback (sess);
+	session_free (sess);
 	done = TRUE;
 }
 
@@ -636,11 +619,11 @@ fe_is_banwindow (struct session *sess)
 	return 0;
 }
 void
-fe_add_ban_list (struct session *sess, char *chan, char *users, char *topic)
+fe_add_ban_list (struct session *sess, char *mask, char *who, char *when, int is_exemption)
 {
-}               
+}
 void
-fe_ban_list_end (struct session *sess)
+fe_ban_list_end (struct session *sess, int is_exemption)
 {
 }
 void
@@ -685,6 +668,10 @@ fe_userlist_clear (struct session *sess)
 {
 }
 void
+fe_userlist_set_selected (struct session *sess)
+{
+}
+void
 fe_dcc_add (struct DCC *dcc)
 {
 }
@@ -788,6 +775,10 @@ fe_set_away (server *serv)
 {
 }
 void
+fe_set_color_paste (session *sess, int status)
+{
+}
+void
 fe_serverlist_open (session *sess)
 {
 }
@@ -812,6 +803,45 @@ fe_gui_info (session *sess, int info_type)
 {
 	return -1;
 }
+void *
+fe_gui_info_ptr (session *sess, int info_type)
+{
+	return NULL;
+}
 void fe_confirm (const char *message, void (*yesproc)(void *), void (*noproc)(void *), void *ud)
 {
 }
+char *fe_get_inputbox_contents (struct session *sess)
+{
+	return NULL;
+}
+void fe_set_inputbox_contents (struct session *sess, char *text)
+{
+}
+int fe_get_inputbox_cursor (struct session *sess)
+{
+	return 0;
+}
+void fe_set_inputbox_cursor (struct session *sess, int delta, int pos)
+{
+}
+void fe_open_url (const char *url)
+{
+}
+void fe_menu_del (menu_entry *me)
+{
+}
+char *fe_menu_add (menu_entry *me)
+{
+	return NULL;
+}
+void fe_menu_update (menu_entry *me)
+{
+}
+void fe_uselect (struct session *sess, char *word[], int do_clear, int scroll_to)
+{
+}
+void
+fe_server_event (server *serv, int type, int arg)
+{
+}
